/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.accservice.adapt;

import org.eclipse.actf.accservice.AccessibilityServiceManager;
import org.eclipse.actf.accservice.IAccessibilityService;
import org.eclipse.actf.accservice.core.IAccessibleElement;
import org.eclipse.actf.core.adapt.IAdaptor;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.IReporter;


/**
 * base type for all accessibility adaptors in ACTF. All accessibility adaptors 
 * extend this class and support adapting a given object to an instance of <code>IAccessibleElement</code>. 
 *
 * @see org.eclipse.actf.core.accessibility.IAccessibleElement
 * @author Mike Squillace
 *
 */
public abstract class AccessibilityAdaptor implements IAdaptor
{

	private static final Class[] DEFAULT_ACCESSIBLE_TYPES = new Class[] {IAccessibleElement.class};

	/** {@inheritDoc} */
	public Class[] getSupportedTypes () {
		return DEFAULT_ACCESSIBLE_TYPES;
	}
	
	protected IAccessibleElement createAccessibleElement (Object element, Object[] params, String accServiceName) {
		IAccessibleElement result = null;
		try {
			IAccessibilityService accService = AccessibilityServiceManager.getInstance().getAccessibilityService(accServiceName);
			if (accService != null) {
				result = accService.createAccessibleElement(element, params);
			}
		}catch (Exception e) {
			Utils.println(IReporter.SYSTEM_NONFATAL, "Error creating accessible element for " + element, e);
		}
		
		return result;
	}
	
}
