/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Kavitha Teegala - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.accservice.core.win32.ia2;

import java.awt.Point;

import org.eclipse.actf.model.InvalidComponentException;


/**
 *
 * @see IA2Accessible
 * @see IA2Accessible#getAccessibleComponent
 * 
 * @author Kavitha Teegala
 */
public class IA2AccessibleComponent
{

	// This reference is used to ensure that the IA2Accessible parent
	// does not go out of scope while this class exists.  When IA2Accessible
	// goes out of scope the accRef will be disposed
	private IA2Accessible _parent = null;

	private int _accRef;

	/**
	 * Constructor used to create an accessible image object
	 * @param image reference pointer to the IA2Accessible image object
	 * @param parent IA2Accessible parent of this object
	 */
	public IA2AccessibleComponent(int val, IA2Accessible parent) {
		_accRef = val;
		_parent = parent;
	}

	/**
	 * used by native code only. Clients should not call directly.
	 * @return ptr address for native object
	 */
	public int internalRef () {
		return _accRef;
	}

	/**
	 * used before each public method call to confirm that
	 * this object is valid and can return correct information
	 * @throws InvalidComponentException
	 */
	private void checkIsValid () throws InvalidComponentException {
		if (_accRef == 0 || _parent == null) { throw new InvalidComponentException("Invalid accessible image"); }
	}

    public Point getLocation() {
    	Point P = null;
    	int[] pArray = internalGetLocation();
    	if(pArray!=null && pArray.length>0){
    		P = new Point(pArray[0], pArray[1]);
    	}
        return P;
    }
	protected native int[] internalGetLocation();
	
    public int getForeground() {

        return internalGetForeground();
    }
	protected native int internalGetForeground();
	
    public int getBackground() {

        return internalGetBackground();
    }
	protected native int internalGetBackground();
}
