/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.accservice.event;

import org.eclipse.actf.accservice.core.IAccessibleElement;
import org.eclipse.actf.model.InvalidComponentException;
import org.eclipse.actf.model.event.ModelEvent;



public class AccessibilityModelEvent extends ModelEvent
{

	public static final String ACCESSIBLE_NAME_PROPERTY = "accessibleName";
	public static final String ACCESSIBLE_ROLE_PROPERTY = "accessibleRole";
	public static final String ACCESSIBLE_STATE_PROPERTY = "accessibleState";

	/**
	 * create an event based on a Windows callback. The source object may be an <code>IAccessibleElement</code> instance
	 * or merely contain information about such an instance.
	 * 
	 * @param source
	 * @see org.eclipse.actf.accservice.core.win32.msaa.MsaaAccessibilityEventService#winEventCallback(Object, int)
	 */
	public AccessibilityModelEvent (Object source) {
		super(source);
		
		if (source instanceof IAccessibleElement) {
			IAccessibleElement acc = (IAccessibleElement) source;
			try {
				setAccessibleName(acc.getAccessibleName());
				setAccessibleRole(acc.getAccessibleRole());
				setAccessibleState((acc.getAccessibleState()).toString());
			} catch (InvalidComponentException e) {
			}
		} else if (source instanceof String[]) {
			String props[] = (String[]) source;
			if(props!=null && props.length==5){
				long timeMillis = Long.parseLong(props[4]);
				setAccessibleName(props[0]);
				setAccessibleRole(props[1]);
				setAccessibleState(props[2]);
				setEventType(props[3]);
				setTimeMillis(timeMillis);
			}
		}
	}

	/**
	 * convenience method for getting the accessible name of the accessible that fired this event
	 * 
	 * @return name of accessible that fired this event
	 */
	public String getAccessibleName () {
		return (String) getProperty(ACCESSIBLE_NAME_PROPERTY);
	}
	
	/**
	 * convenience method for setting the accessible name of the accessible that fired this event. This event should nt 
	 * be called by clients. It is made public for use by native code capturing the event.
	 *  
	 * @param accName
	 */
	public void setAccessibleName (String accName) {
		setProperty(ACCESSIBLE_NAME_PROPERTY, accName);
	}
	
	/**
	 * convenience method for getting the accessible role of the accessible that fired this event
	 * 
	 * @return role of accessible that fired this event
	 */
	public String getAccessibleRole () {
		return (String) getProperty(ACCESSIBLE_ROLE_PROPERTY);
	}
	
	/**
	 * convenience method for setting the accessible role of the accessible that fired this event. This event should nt 
	 * be called by clients. It is made public for use by native code capturing the event.
	 *  
	 * @param accRole
	 */
	public void setAccessibleRole (String accRole) {
		setProperty(ACCESSIBLE_ROLE_PROPERTY, accRole);
	}

	public void setAccessibleState (String accState) {
		setProperty(ACCESSIBLE_STATE_PROPERTY, accState);
	}
	
	public String getAccessibleState () {
	 Object state = getProperty(ACCESSIBLE_STATE_PROPERTY);
	 return state.toString();
	}
	

}