/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  wscarter - initial API and implementation
*******************************************************************************/ 
/**
 * 
 */
package org.eclipse.actf.accservice.javapp;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.actf.accservice.IAccessibilityEventService;
import org.eclipse.actf.accservice.IWindowService;
import org.eclipse.actf.model.event.IModelEventListener;


/**
 * @author wscarter
 *
 */
public class JavaAccessibilityEventService implements IAccessibilityEventService
{
	protected static Map _listenerMap = new HashMap();
	protected Map _hookMap = new HashMap();
	protected IWindowService _windowService;
	
	public JavaAccessibilityEventService (IWindowService windowService) {
		_windowService = windowService;
	}
	/**
	 * add the listener to be notified when events of the given type are fired from the underlying system
	 * 
	 * @param listener
	 * @param eventTypes to be chosen from constants in this class
	 * @see org.eclipse.actf.accservice.win32.msaa.MsaaGuiModel#registerModelEventListener(IModelEventListener, Object[])
	 */
	public void addAccessibilityEventListener(IModelEventListener listener, Object[] eventTypes) {
		
		for (int i = 0; i < eventTypes.length; ++i) {
			Integer eventId = (Integer) eventTypes[i];
			List listeners = (List) _listenerMap.get(eventId);
			if (listeners == null) {
				int processId = _windowService.getProcessId(_windowService.getActiveWindow());
				// int hook = setWinEventHook(eventId.intValue(), eventId.intValue(), processId, 0);
				int hook = 1; // fix???
				if (hook != -1) {
					listeners = new LinkedList();
					_listenerMap.put(eventId, listeners);
					// _hookMap.put(eventId, new Integer(hook));
				}
			}
			listeners.add(listener);
		}
	}
	
	/** {@inheritDoc} */
	public void removeAccessibilityEventListener(IModelEventListener listener, Object[] eventTypes) {
		for (int i = 0; i < eventTypes.length; ++i) {
			Integer eventId = (Integer) eventTypes[i];
			List listeners = (List) _listenerMap.get(eventId);
			if (listeners != null) {
				listeners.remove(listener);
				if (listeners.isEmpty()) {
					_listenerMap.remove(eventId);
					// removeWinEventHook(((Integer) _hookMap.get(eventId)).intValue());
					_hookMap.remove(eventId);
				}
			}
		}
	}
}
