/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Smith - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.accservice.javapp;

import javax.accessibility.AccessibleIcon;

import org.eclipse.actf.accservice.core.IAccessibleImageElement;
import org.eclipse.actf.model.InvalidComponentException;

/**
 * implementation of the <code>IAccessibleImageElement</code> object for the Java Accesibility API's <code>AccessibleIcon</code>.
 * 
 * @see javax.accessibility.AccessibleIcon
 * @author Mike Smith
 */
public class JavaAccessibleImage implements IAccessibleImageElement
{

	private AccessibleIcon _image;

	/**
	 * Constructor used to create an accessible image object. This constructor will usually 
	 * be invoked from <code>JavaAccessible</code> and is not typically called by clients.
	 * 
	 * @param image -
	 *            AccessibleIcon object
	 */
	public JavaAccessibleImage (AccessibleIcon image) {
		_image = image;
	}

	private void checkIsValid () throws InvalidComponentException {
		if (_image == null) { throw new InvalidComponentException("No accessible context"); }
	}

	/** {@inheritDoc} */
	public String getDescription () throws InvalidComponentException {
		checkIsValid();
		return _image.getAccessibleIconDescription();
	}

	/** {@inheritDoc} */
	public int getHeight () throws InvalidComponentException {
		checkIsValid();
		return _image.getAccessibleIconHeight();
	}

	/** {@inheritDoc} */
	public int getWidth () throws InvalidComponentException {
		checkIsValid();
		return _image.getAccessibleIconWidth();
	}
}
