/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Smith - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.accservice.javapp;

import javax.accessibility.AccessibleValue;

import org.eclipse.actf.accservice.core.IAccessibleValueElement;
import org.eclipse.actf.model.InvalidComponentException;

/**
 * implementation of the <code>IAccessibleImageElement</code> object for the Java Accesibility API's <code>AccessibleIcon</code>.
 * 
 * @see javax.accessibility.AccessibleIcon
 * @author Mike Smith
 */
public class JavaAccessibleValue implements IAccessibleValueElement
{

	private AccessibleValue _value;

	/**
	 * Constructor used to create an accessible image object. This constructor will usually 
	 * be invoked from <code>JavaAccessible</code> and is not typically called by clients.
	 * 
	 * @param image -
	 *            AccessibleIcon object
	 */
	public JavaAccessibleValue (AccessibleValue value) {
		_value = value;
	}

	private void checkIsValid () throws InvalidComponentException {
		if (_value == null) { throw new InvalidComponentException("No accessible context"); }
	}

	public Object getCurrentValue() throws InvalidComponentException {
		checkIsValid();
		return _value.getCurrentAccessibleValue();
	}

	public Object getMaximumValue() throws InvalidComponentException {
		checkIsValid();
		return _value.getMaximumAccessibleValue();
	}

	public Object getMinimumValue() throws InvalidComponentException {
		checkIsValid();
		return _value.getMinimumAccessibleValue();
	}

	public void setCurrentValue(Object value) throws InvalidComponentException {
		checkIsValid();
		if( value instanceof Number) {
			Number n = (Number)value;
			_value.setCurrentAccessibleValue(n);
		}
		
	}
	
}
