/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.model.dom.locate;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.jxpath.ri.model.NodePointerFactory;
import org.apache.commons.jxpath.ri.model.dom.DOMPointerFactory;
import org.eclipse.actf.model.dom.DomModel;
import org.eclipse.actf.model.locate.AbstractNodeLocator;
import org.w3c.dom.Node;


/**
 * a locator for elements of a W3C DOM-compliant hierarchy
 * 
 * @author Mike Squillace
 *
 */
public class DomNodeLocator extends AbstractNodeLocator
{
	 
		/**
	 * create an element locator for W3C DOM elements
	 *
	 */
	public DomNodeLocator () {
		super("id", DomModel.DOM_MODEL, new DOMPointerFactory());
		propertiesMap = initializeProperties();
	}
	
	protected DomNodeLocator (String idPropName, String modelName, NodePointerFactory factory) {
		super(idPropName, modelName, factory);
		propertiesMap = initializeProperties();
	}
	
	private Map initializeProperties() {
		Map m = new HashMap();
		m.put(org.w3c.dom.Element.class.getName(), "id");
		return m;
	}
	public String locate (Object element, Object root) {
        StringBuffer sb = new StringBuffer();
        
        if (element != null && element instanceof Node) {
        	Node node = (Node) element;
        	Node rootNode = null;
        	if (root == null) {
        		rootNode = node.getOwnerDocument();
        	} else if (root instanceof Node) {
        		rootNode = (Node) root;
        	}
        	
        	while (node != null && !node.equals(rootNode)) {
        		if (node.getNodeType() == Node.ELEMENT_NODE) {
        			prependNodeSegment(sb, node);
        		} else if (node.getNodeType() == Node.TEXT_NODE || node.getNodeType() == Node.CDATA_SECTION_NODE) {
        			sb.append("/text()");
        		}
        		node = node.getParentNode();
        	}
    }
        
        return sb.toString();
	}
	

	protected void prependNodeSegment (StringBuffer buffer, Node node) {
		StringBuffer sb = new StringBuffer();
		
		if (sb.length() == 0 || sb.charAt(sb.length() - 1) != '/') {
            sb.append('/');
        }
		
        String uri = node.getNamespaceURI();
        String name = node.getLocalName();
        if (uri == null) {
            sb.append(name);
            sb.append('[');
            sb.append(getRelativePositionByName(node));
            sb.append(']');
        } else {
            String prefix = node.getPrefix();
            if (prefix != null) {
                sb.append(prefix);
                sb.append(':');
                sb.append(name);
                sb.append('[');
                sb.append(getRelativePositionByName(node));
                sb.append(']');
            }
        }
        
        buffer.insert(0, sb.toString());
	}
	
	protected int getRelativePositionByName(Node node ) {
		int count = 1;
		Node prev = node.getPreviousSibling();
		while (prev != null) {
			if (prev.getNodeType() == Node.ELEMENT_NODE && prev.getNodeName().equals(node.getNodeName())) {
				++count;
			}
			prev = prev.getPreviousSibling();
		}

		return count;
	}

}
