/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.model.javapp.locate;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.eclipse.actf.model.javapp.SwtGuiModel;
import org.eclipse.actf.model.locate.AbstractNodeLocator;


/**
 * @author Mike Squillace
 */
public class SwtComponentLocator extends AbstractNodeLocator
{

	/**
	 * create a new SwtComponentLocator. By default, the components that can be
	 * identified via the <code>identify</code> method are found in the
	 * 'swt.xml' file.
	 */
	public SwtComponentLocator () {
		super("id", SwtGuiModel.SWT_MODEL, new SwtNodePointerFactory());
		propertiesMap = initializeProperties();
	}

	private Map initializeProperties() {
		Map m = new HashMap();
		m.put(org.eclipse.swt.widgets.Button.class.getName(), "text");
		m.put(org.eclipse.swt.widgets.Label.class.getName(), "text");
		m.put(org.eclipse.swt.custom.CCombo.class.getName(), "text");
		m.put(org.eclipse.swt.widgets.Combo.class.getName(), "text");
		m.put(org.eclipse.swt.widgets.Shell.class.getName(), "text");
		m.put(org.eclipse.swt.widgets.Item.class.getName(), "text");    
	   	return m;
	}
	
	
	/**
	 * identify or distinguish the SWT Widget in an SWT component hierarchy. This method will first attempt 
	 * to identify the given control using <code>super.identify(Object)</code>. Next, if the <code>getData</code> method 
	 * exists, it will be called with the value returned by <code>getDefaultIdPropertyName()</code>.
	 *
	 * @see AbstractNodeLocator#describe(Object)
	 * @see AbstractNodeLocator#getDefaultIdPropertyName()
	 */
	public Properties describe (Object element) {
		Properties result = super.describe(element);
		if (result == null) {
			try {
				result = new Properties();
				result.setProperty(
					getDefaultIdPropertyName(),
					(String) ((SwtGuiModel) model).invokeGetter(
					element, "getData", getDefaultIdPropertyName()));
			}catch (Exception e) {
			}
		}
		
		return result;
	}
	
} // SwtComponentLocator
