/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* Ann Ford - initial API and implementation
*******************************************************************************/ 
package org.eclipse.actf.model.javapp.traverse;

import java.awt.Component;
import java.util.HashSet;

import javax.accessibility.AccessibleContext;
import javax.accessibility.AccessibleRole;

import org.eclipse.actf.model.traverse.filters.INodeFilter;

/**
 * This filter will exclude items.  
 * If you wish to show all items,
 * remove it, or do not set it in the first place.
 * @author annford
 *
 */
public class SwingNodeItemFilter implements INodeFilter
{
	
	protected static final HashSet ITEM_ROLES = new HashSet();
	static {
		ITEM_ROLES.add(AccessibleRole.LIST_ITEM);
		ITEM_ROLES.add(AccessibleRole.MENU_ITEM);
	}
	
	public boolean pass (Object node) {
		boolean item = false;
		if (node != null && node instanceof Component) {
			Component component = (Component)node;
			AccessibleContext accContext = component.getAccessibleContext();
			item = accContext != null && ITEM_ROLES.contains(accContext.getAccessibleRole());
		}
		return !item;
	}
}
