/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

import java.util.HashMap;
import java.util.Map;

/**
 * default factory for mapping validators to validation processes. A factory
 * instance, once obtained, can be used to find the validation process class
 * name for a given validator.
 * 
 * <p>
 * <b>Note</b>: Mappings are currently hard-coded. Future work incdlues
 * permitting specification of these mappings in the validation document
 * rulebases.
 * 
 * 
 * @author John Buslawski
 * 
 */
public class DefaultValidationProcessMappingFactory implements
		IValidationProcessMappingFactory {

	protected static final Map VALIDATION_PROCESS_MAP = new HashMap();
	static {
		VALIDATION_PROCESS_MAP.put(
						"org.eclipse.actf.validation.javapp.validators.SwtAccessibilityValidator",
						"org.eclipse.actf.validation.core.AccessibilityValidationProcess");
		VALIDATION_PROCESS_MAP.put(
						"org.eclipse.actf.validation.javapp.validators.JavaAccessibilityValidator",
						"org.eclipse.actf.validation.core.AccessibilityValidationProcess");
		VALIDATION_PROCESS_MAP.put(
						"org.eclipse.actf.validation.mozdom.validators.MozillaDomAccessibilityValidator",
						"org.eclipse.actf.validation.core.AccessibilityValidationProcess");
		VALIDATION_PROCESS_MAP.put(
						"org.eclipse.actf.validation.validators.SwtFocusValidator",
						"org.eclipse.actf.validation.core.FocusValidationProcess");
		VALIDATION_PROCESS_MAP.put(
				"org.eclipse.actf.validation.validators.DomAccessibilityValidator",
				"org.eclipse.actf.validation.core.AccessibilityValidationProcess");
		VALIDATION_PROCESS_MAP.put(
						"org.eclipse.actf.validation.validators.GenericAccessibilityValidator",
						"org.eclipse.actf.validation.core.AccessibilityValidationProcess");
	}

	// a handle to the singleton ReflectionDocumentRuleFactory
	static private DefaultValidationProcessMappingFactory instance = null;

	/**
	 * @return The unique instance of this class.
	 */
	static public DefaultValidationProcessMappingFactory getInstance() {
		if (null == instance) {
			instance = new DefaultValidationProcessMappingFactory();
		}
		return instance;
	}

	protected DefaultValidationProcessMappingFactory() {
	}

	/** {@inheritDoc} */
	public String getValidationProcessMapping(String name) {
		return (String) VALIDATION_PROCESS_MAP.get(name);
	}

	/** {@inheritDoc} */
	public Map getValidationProcessMappings() {
		return VALIDATION_PROCESS_MAP;
	}

	public void registerValidationProcessMapping(String validatorName,
			String validationProcessName) {
		if (validatorName == null || validatorName.length() == 0) {
			return;
		}
		if (validationProcessName != null) {
			if (!VALIDATION_PROCESS_MAP.containsKey(validatorName)) {
				VALIDATION_PROCESS_MAP.put(validatorName, validationProcessName);
			}
		}
	}
}
