/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.doc.rulebase;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.actf.validation.rules.IRuleMapping;


public class ValidationDocumentRuleMapping implements IRuleMapping {

	private Map componentRuleMap;
	private Set polymorphicRuleTypes;
	private Set nonPolymorphicRuleTypes;

	public ValidationDocumentRuleMapping(Map ruleMap, Set polyRuleTypes, Set nonPolyRuleTypes) {
		componentRuleMap = ruleMap;
		polymorphicRuleTypes = polyRuleTypes;
		nonPolymorphicRuleTypes = nonPolyRuleTypes;
	}

	protected ValidationDocumentRuleMapping() {
		componentRuleMap = new HashMap();
		polymorphicRuleTypes = new HashSet();
		nonPolymorphicRuleTypes = new HashSet();
	}

	public Set getNonPolymorphicRuleTypes() {
		return nonPolymorphicRuleTypes;
	}

	public Set getPolymorphicRuleTypes() {
		return polymorphicRuleTypes;
	}

	public void setNonPolymorphicRuleTypes(Set ruleTypes) {
		nonPolymorphicRuleTypes = ruleTypes;
	}

	public void setPolymorphicRuleTypes(Set ruleTypes) {
		polymorphicRuleTypes = ruleTypes;
	}

	public Map getRules() {
		return componentRuleMap;
	}

	public List getRulesForComponent(String id, Object component) {
		List rulesList = new LinkedList();
		// if id appears in ruleMap, then
		// a set of rules to be validated is associated with this id
		if (id != null) {
			rulesList.add(componentRuleMap.get(id));
		}
		// need to find the classes in the class sets of which this
		// component is an instance;
		// for each class of which this component is an instance, add
		// the corresponding validation rules to be invoked
		Class compClass = component.getClass();
		List interfaces = Arrays.asList(compClass.getInterfaces());
		if (nonPolymorphicRuleTypes.contains(compClass)) {
			rulesList.add(componentRuleMap.get(compClass.getName()));
		} else {
			for (Iterator iter = polymorphicRuleTypes.iterator(); iter.hasNext();) {
				Class setElem = (Class) iter.next();
				if (setElem.isAssignableFrom(compClass)
						|| interfaces.contains(setElem)) {
					rulesList.add(componentRuleMap.get(setElem.getName()));
				}
			}
		}
		return rulesList;
	}

	public void setRules(Map rules) {
		componentRuleMap = rules;
	}

	public List getComponentCriteria(Object component) {
		List rulesList = new LinkedList();
		// need to find the classes in the class sets of which this
		// component is an instance;
		// for each class of which this component is an instance, add
		// the corresponding validation rules to be invoked
		Class setElem = null, compClass = component.getClass();
		if (nonPolymorphicRuleTypes.contains(compClass)) {
			rulesList.add(componentRuleMap.get(compClass.getName()));
		} else {
			Iterator iter = polymorphicRuleTypes.iterator();
			while (iter.hasNext()) {
				setElem = (Class) iter.next();
				if (setElem.isAssignableFrom(compClass)) {
					rulesList.add(componentRuleMap.get(setElem.getName()));
				}
			}
		}
		return rulesList;
	}
	
}
