/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.doc.scripting;

import org.eclipse.actf.model.IModel;
import org.mozilla.javascript.Context;

/**
 * A code interpreter for scripts written in JavaScript/Rhino.
 * 
 * @author Mike Squillace
 */
public class JavaScriptCodeInterpreter extends AbstractCodeInterpreter {

	/**
	 * create a new interpreter for JavaScript
	 * 
	 * @param lang
	 * @param arch
	 */
	public JavaScriptCodeInterpreter(String lang, IModel model) {
		super(lang, model);

		if (Context.getCurrentContext() == null) {
			Context.enter();
		}
	}

	protected void initClasspath() {
		Context.getCurrentContext().setApplicationClassLoader(getClass().getClassLoader());
	}

	protected String formatPackageImport (String packageName) {
		return "importPackage(Packages." + packageName + ")\n";
	}
	
	/** {@inheritDoc} */
	protected Object toJavaObject(Object o, Class c) {
		return o;
	}

	/** {@inheritDoc} */
	protected Object[] toJavaObjects(Object o, Class[] paramTypes) {
		return new Object[] { o };
	}

	protected void finalize() throws Throwable {
		try {
			Context.exit();
		} finally {
			super.finalize();
		}
	}

} // JavaScriptCodeInterpreter
