/**
 * 
 */
package org.eclipse.actf.validation.javapp;

import java.io.File;
import java.util.Vector;

import org.eclipse.actf.model.javapp.SwingGuiModel;
import org.eclipse.actf.model.javapp.SwtGuiModel;
import org.eclipse.actf.validation.core.ExecutionPoint;
import org.eclipse.actf.validation.core.ValidationProcessException;
import org.eclipse.actf.validation.javapp.aspects.AspectGenerator;
import org.eclipse.actf.validation.rulebase.IRuleBase;
import org.eclipse.actf.validation.validators.IValidationContext;

/**
 * @author administrator
 *
 */
public class AspectGenerationProcess 
{
	public static final String INTRAJVM = "intra";
	public static final String INTERJVM = "inter";
	//indentation
	private static final String indent = AspectGenerator.indent;
	//line separator
	private static final String  div = System.getProperty("line.separator");
	
	AspectGenerator generator;
	
	//swt, swing
	private String modelType;
	
	//intra- or inter-jvm
	private String jvmMode;
	
	//fully qualified name (including package) of the source (intra) or class (inter) to be validated 
	private String fullTargetName;
	
	private String workingDir;
	
	public void setModelType(String modelType) {
	
		this.modelType = modelType;
	}
	
	public void setJvmMode(String jvmMode) {
		
		 this.jvmMode = jvmMode;	
	}	
	
	public void setFullTargetName(String fullTargetName)  {
		
	    this.fullTargetName = fullTargetName;	
	}	
	
	public void setWorkingDir(String workingDir) {
		
		this.workingDir = workingDir;
	}
	
	/**
	 * @param ruleBase
	 * @param execPoint
	 * @param context
	 */
	public AspectGenerationProcess (IRuleBase ruleBase, ExecutionPoint execPoint,
									IValidationContext context) {
		
		generator = new AspectGenerator();
		// TODO Auto-generated constructor stub
	}

	/**
	 * 
	 */
	public AspectGenerationProcess () {
		// TODO Auto-generated constructor stub
		generator = new AspectGenerator(); 
	}
	
	public void generate(String packageName, String aspectName, String executableName, String workingDir, String jvmMode, String modelType)  {
		
		 
	     setFullTargetName(executableName);
	     setWorkingDir(workingDir);
	     setJvmMode(AspectGenerationProcess.INTERJVM);
	     setModelType("swt");
	     
	     /** Load ajde into the ClassLoaderCache
			ClassLoader loader = getClass().getClassLoader();
			Bundle swtBundle = Platform.getBundle("org.aspectj.ajde");
			
			Class c = null;
			if (swtBundle != null) {			
				
				try {
				   c = swtBundle.loadClass("org.aspectj.ajdt.ajc.AjdtCommand"); 
				   //clCache.put( swtBundle.getSymbolicName(), c.getClassLoader());
				} catch (Exception ex) {
					
				}
			}
	     **/
	     /**
	     Class cls = null;
	     ClassLoader clsl = null;
	     ClassLoader loader = null;
	     ClassLoader parent = null;
	     try {
	         cls = Class.forName("org.aspectj.ajdt.ajc.AjdtCommand");
	         loader = cls.getClassLoader();
	         clsl = this.getClass().getClassLoader();
	         parent = clsl.getParent();
	     } catch ( Exception exc) {
	        	 
	     }
	     */
	     try {
		    init(packageName, aspectName);
		    File file = generator.generateAspectFile(workingDir, aspectName);
		    generator.compileAspectFile(workingDir, file);
		    	
	     } catch (Exception exc) {
	       exc.getMessage();
	     }
   }

	public void init(String packageName, String aspectName) throws ValidationProcessException {
		
		  generateImports();
		  generateAspect(packageName, aspectName);
		  generateIsActfPointcut();
		  generateValidationPointcut();
		  generateInstantiationPointcut();
		  generateValidationAdvice();
		  generateInstantiationAdvice("constructor_call");
		  generateInstantiationAdvice("constructor_execution");
		  
		  if (jvmMode.equals(INTRAJVM)) { //intra
			  generateTerminationPointcut();
			  generateTerminationAdvice();
		  }
		  
		  
	}
	
	private void generateImports() throws ValidationProcessException {
		
	     Vector imports =  new Vector();
	     imports.addElement("org.aspectj.lang.JoinPoint");
	     imports.addElement("org.aspectj.lang.Signature");
	     imports.addElement("org.eclipse.actf.model.javapp.SwtGuiModel");
	     imports.addElement("org.eclipse.actf.model.javapp.SwingGuiModel");
	     imports.addElement("org.eclipse.actf.validation.core.ValidationProcessException");
	     imports.addElement("javax.swing.JTabbedPane");
	     imports.addElement("org.eclipse.actf.validation.javapp.JavaValidationConstants");
	     
	     generator.addImports(imports);
	     
	}
	
	private void generateAspect(String packageName, String name) throws ValidationProcessException {
		
		//String packageName = "org.eclipse.actf.validation.javapp.aspects";
		
		String superClass = null;
		if (jvmMode.equals(INTRAJVM))  {
			//aspectName = "GenIntraJvmValidationAspect";
		    superClass = "IntraJvmValidation";
		    
		} else { //inter-jvm
			//aspectName ="GenInterJvmValidationAspect";
			superClass = "InterJvmValidationAspect";
		}
		
		generator.createAspect(packageName, name, superClass); 
		
	}
	
	private void generateIsActfPointcut() throws ValidationProcessException {
		
		String name = "isActfType";
		String parameterList = "";
		String expression = "within(org.eclipse.actf..*);";
		
		generator.createPointcut(name, parameterList, expression.toString()); 
		
	}
	
	
	private void generateValidationPointcut() throws ValidationProcessException {
		
		String name = "validationPointcut";
		String parameterList = "Object component";
		
		StringBuffer expression = new StringBuffer();
		
		expression.append("within("+ fullTargetName +") && (");
		expression.append(div);
		expression.append(indent);
		if (modelType.equals(SwtGuiModel.SWT_MODEL)) {
		
			if (jvmMode.equals(INTRAJVM)) {
			   
		       expression.append("(target(Shell) && target(component) && call(public void *.open()))");
		       expression.append("|| (target(Control) && target(component) && call(public void *.setVisible(boolean)))");
			
			} else  {  //inter
				
				expression.append("target(Control) && target(component) ");
				expression.append("&& call(public void *.setVisible(boolean))");
			}
			
		} if (modelType.equals(SwingGuiModel.SWING_MODEL)) {  
			
		    expression.append("target(Component) && target(component)");
		    expression.append("&& (call(public void *.setVisible(boolean)) || call(public void *.show())");
		    expression.append("|| call(public void javax.swing.JTabbedPane.setSelectedIndex(int)))");
		}
		
		expression.append(div +indent+ ");");
				
		generator.createPointcut(name, parameterList, expression.toString());   	
	    	
	}
	  
	private void generateInstantiationPointcut() throws ValidationProcessException {
		
		String name = "instantiationPointcut";
		String parameterList = "Object newObj";
		
		StringBuffer expression = new StringBuffer();
		
		expression.append(indents(2));
		expression.append("within("+ fullTargetName +") && (");
		expression.append(div);
		
        if (modelType.equals(SwtGuiModel.SWT_MODEL)) {
        	expression.append("(this(Widget) && this(newObj) && initialization(*.new(..)))");
			
			if (jvmMode.equals(INTRAJVM)) { //intra
					
			   expression.append("|| (this(Object) && this(newObj) && call(org.eclipse.swt.widgets.*.new(..)))");
			}
			
		} if (modelType.equals(SwingGuiModel.SWING_MODEL)) {  
			expression.append(indents(2));
		    expression.append("(this(JComponent) && this(newObj) && initialization(*.new(..)))");
		    expression.append("|| (this(Object) && this(newObj) && call(javax.swing.J*.new(..)))");
		    
		}
		
		expression.append(div + indent + ");");
				
		generator.createPointcut(name, parameterList, expression.toString());
						
	}
	
	private void generateTerminationPointcut() throws ValidationProcessException {
		
		String name = "terminationPointcut";
		String parameterList = "";
		
		StringBuffer expression = new StringBuffer();
		
		expression.append(indents(2));
		expression.append("within("+ fullTargetName +") && (");
		
        if (modelType.equals(SwtGuiModel.SWT_MODEL)) {
        	expression.append(indents(2));	
			expression.append("(this(Widget) && this(newObj) && initialization(*.new(..)))");
			expression.append("|| (this(Object) && this(newObj) && call(org.eclipse.swt.widgets.*.new(..)))");
			
						
		} if (modelType.equals(SwingGuiModel.SWING_MODEL)) {  
			expression.append(indents(2));
			expression.append("call(public void java.lang.System.exit(int))");
					    
		}
		
		expression.append(");");
				
		generator.createPointcut(name, parameterList, expression.toString());
		
		
	}
	
	private void generateValidationAdvice() throws ValidationProcessException {
		
		String type = "after";
		String parameterList = "Object component";
		
		String afterAction = "returning";
		String pointcutExpression = "!isActfType() && validationPointcut(component)";
		
		StringBuffer body = new StringBuffer();
		body.append("Signature sig = thisJoinPoint.getSignature();" + div);
		body.append(indents(2));
		body.append("String name = sig.getName();"  + div);
		body.append(indents(2));
		body.append("String clsName = sig.getDeclaringTypeName();"  + div);
		
		if (jvmMode.equals(INTERJVM)) {
			body.append(indents(2));
		    body.append("Object[] args = thisJoinPoint.getArgs();" + div);
		    body.append(indents(2));
		    body.append("boolean valid=");
		    
		    if (modelType.equals(SwtGuiModel.SWT_MODEL)) {  
		    
		         body.append("(args.length != 1 || (args[0] instanceof Boolean && ((Boolean)args[0]).booleanValue()));" + div);
		    
		    } else if (modelType.equals(SwingGuiModel.SWING_MODEL)) { 
		    
			     body.append("(name.equals(\"show\") || name.equals(\"setSelectedIndex\")");
     		     body.append("|| (name.equals(\"setVisible\") && ((Boolean)args[0]).booleanValue()));");
		    } else {
		    	//shoudn't get here
		    	throw new ValidationProcessException("The specified model is not supported");
		    }
		    
     		body.append(div+indents(2));
			body.append("if (valid) {"); 
			body.append(div + indents(3));		
					
			body.append("if (sig.getName().equals(\"setSelectedIndex\")) {");
			body.append(div + indents(4));
			body.append("component = ((JTabbedPane) component).getComponentAt(");
			body.append("((Integer) thisJoinPoint.getArgs()[0]).intValue());");
			body.append(div +indents(4));
			body.append("clsName = component.getClass().getName();");
			body.append(div + indents(3));
			body.append("}");		
			body.append(div + indents(2));
			body.append("}");
		
			body.append(div + div +indents(2));
		    body.append("StringBuffer reportID = new StringBuffer(clsName);");
		    body.append(div + indents(2));
		    body.append("reportID.append('@');");
		    body.append(div + indents(2));
		    body.append("reportID.append(System.identityHashCode(component));");
		    body.append(div +indents(2));
		    body.append("reportID.append('.');");
		    body.append(div +indents(2));
		    body.append("reportID.append(sig.getName());");
		    
		    if (jvmMode.equals(INTERJVM)) {
		    	
		    	body.append(div + div +indents(2));
		    	body.append("setReportID(reportID.toString());");
		    	body.append(div + indents(2));
			    body.append("startValidationThread(\"component\");");
		    }
		}
		
		generator.createAdvice(type, parameterList, pointcutExpression, body.toString(), afterAction);
	}
	
	private void generateInstantiationAdvice(String kind) throws ValidationProcessException {
		
		String type = "after";
		String parameterList = "Object obj";
		String afterAction = "returning";
		
		if (kind.equals("constructor_call")) {
		    afterAction = "returning(Object newObj)";
		} 
		
		String pointcutExpression = "!isActfType() && instantiationPointcut(obj)";
		
		StringBuffer body = new StringBuffer();
		body.append(indents(2));
		body.append("String kind = thisJoinPoint.getKind();" + div);
		body.append(indents(2));
		
		if (kind.equals("constructor_call")) {
		    body.append("if (kind.equals(JoinPoint.CONSTRUCTOR_CALL)) {"  + div);
		    body.append(indents(3));
		    body.append("cacheCreationContext(newObj, thisJoinPoint.getSignature(), new Exception());");
		    
		} else {
			body.append("if (kind.equals(JoinPoint.CONSTRUCTOR_EXECUTION) || kind.equals(JoinPoint.INITIALIZATION)) {"  + div);
			body.append(indents(3));
			body.append("cacheCreationContext(obj, thisJoinPoint.getSignature(), new Exception());");
		}
		
		body.append(div + indents(2));
		body.append("}");
				
		generator.createAdvice(type, parameterList, pointcutExpression, body.toString(), afterAction);
		
	}
	
	private void generateTerminationAdvice() throws ValidationProcessException {
		
		String type = "before";
		String parameterList = "";
		String afterAction = null;
		
		String pointcutExpression = "terminationPointcut()";
		
		StringBuffer body = new StringBuffer();
		body.append(indents(2));
		body.append("if (System.getProperty(JavaValidationConstants.RESOURCE_SERVER_PORT_KEY) == null) {" + div);
		body.append(indents(3));
		
		body.append("throw new IllegalTerminationError(\"Cannot permit termination method - end session from view toolbar\");"  + div);
		body.append(indents(3));
		body.append("}");
		    
		generator.createAdvice(type, parameterList, pointcutExpression, body.toString(), afterAction);
	}
	
	private String indents(int size) {
		
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < size; i++)  {
			sb.append(indent);
		}
		
		return sb.toString();
	}
	
	/**
	 * @param args
	 */
	public static void main (String[] args) {
		
		AspectGenerationProcess agen = new AspectGenerationProcess();
		
		String workingDir = System.getProperty("java.io.tmpdir") + "actf" + File.separator;
		agen.generate("org.eclipse.actf.validation.javapp.aspects", "MyInterAspect", "austin.test.WebTest.Test", workingDir, INTRAJVM, "swt");
		
	}
}
