/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 
package org.eclipse.actf.validation.javapp;

import org.eclipse.actf.core.ActfCorePlugin;
import org.eclipse.actf.validation.ValidationCorePlugin;
import org.eclipse.core.runtime.Platform;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

/**
 * The JavaValidationPlugin class controls the plug-in life cycle
 */
public class JavaValidationPlugin extends ValidationCorePlugin {
	public static final String JAVA_VALIDATION_CORE_JAR =  "actf-valjava.jar";
	public static final String ACTF_ASPECTS_JAR =  "actf-aspects.jar";
	public static final String MODEL_JAVA_UI_JAR =  "modeljavaui.jar";
		
	public static final String ASPECTJ_RUNTIME_BUNDLE = "org.aspectj.runtime";
	public static final String ASPECTJ_RUNTIME_JAR = "aspectjrt.jar";
	public static final String ASPECTJ_WEAVER_BUNDLE = "org.aspectj.weaver";
	public static final String ASPECTJ_WEAVER_JAR = "aspectjweaver.jar";

	
	// The plug-in ID

	public static final String MODELJAVACORE_PLUGIN_ID = "org.eclipse.actf.validation.javapp";

	// The shared instance
	private static JavaValidationPlugin plugin;

	/**
	 * The constructor
	 */
	public JavaValidationPlugin() {
		super();
		plugin = this;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.Plugins#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		
		// Load SWT into the ClassLoaderCache
		Bundle swtBundle = Platform.getBundle("org.eclipse.swt");
		if (swtBundle != null) {			
			Class c = swtBundle.loadClass("org.eclipse.swt.widgets.Display"); 
			clCache.put( swtBundle.getSymbolicName(), c.getClassLoader());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.Plugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 * 
	 * @return the shared instance
	 */
	public static ActfCorePlugin getDefault() {
		return plugin;
	}

	public String getPluginId() {
		return MODELJAVACORE_PLUGIN_ID;
	}

}

