/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.agent;

import java.lang.instrument.ClassFileTransformer;
import java.lang.instrument.IllegalClassFormatException;
import java.security.ProtectionDomain;

import org.aspectj.weaver.loadtime.Aj;
import org.aspectj.weaver.loadtime.ClassPreProcessor;
import org.eclipse.actf.util.resources.ClassLoaderCache;

public class AspectJTransformer implements ClassFileTransformer
{

	private static ClassPreProcessor preProcessor;
	static {
		try {
			preProcessor = new Aj();
			preProcessor.initialize();
		}catch (Exception e) {
			throw new ExceptionInInitializerError("could not initialize JSR163 preprocessor due to: "
					+ e.toString());
		}
	}

	private ClassLoaderCache _clCache = ClassLoaderCache.getDefault();

	private boolean _registered = false;

	public byte[] transform (ClassLoader loader, String className,
								Class classBeingRedefined,
								ProtectionDomain protectionDomain, byte[] bytes)
		throws IllegalClassFormatException {
		byte[] res = bytes;
		String clClassname = className.replace('/', '.');
		if (classBeingRedefined == null
				&& !_clCache.isNonAdvisableClassName(clClassname)) {
			res = preProcessor.preProcess(className, bytes, loader);
//			if (!_registered) {
//				_registered = true;
//				System.setProperty(
//					"org.aspectj.weaver.loadtime.configuration", "<none>");
//			}
		}
		_clCache.put(clClassname, loader);
		return res;
	}
}
