/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.aspects;

import javax.swing.JTabbedPane;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.Signature;

/**
 * aspects used by the AspectJ validation engine
 * 
 */
public abstract aspect AspectJValidationAspect extends InterJvmValidationAspect
{

	public AspectJValidationAspect (String modelName) {
		super(modelName);
		// prevent cacheing of ClassLoaders by AspectJ engine during script-code
		// evaluations of expressions
		System.setProperty("org.aspectj.weaver.loadtime.configuration","<none>");
	}
	
	protected abstract boolean shouldValidate (JoinPoint jp);
	
	// TODO necessary until find out why aop.xml files
	// are not found on classpath in 1.4 mode
	pointcut isActfType(): within(org.eclipse.actf..*);
	
	abstract pointcut validationPointcut(Object component);
	after(Object component) returning: !isActfType() && validationPointcut(component) {
		if (shouldValidate(thisJoinPoint)) {
			Signature sig = thisJoinPoint.getSignature();
			String clsName = null;
			
			// TODO will  be removed once user can specify
			// validation triggers
			if (sig.getName().equals("setSelectedIndex")) {
				component = ((JTabbedPane) component).getComponentAt(
					((Integer) thisJoinPoint.getArgs()[0]).intValue());
				clsName = component.getClass().getName();
			} else {
				clsName = sig.getDeclaringTypeName();
			}
			
			StringBuffer reportID = new StringBuffer(clsName);
			reportID.append('@');
			reportID.append(System.identityHashCode(component));
			reportID.append('.');
			reportID.append(sig.getName());
			setReportID(reportID.toString());
			
			startValidationThread(component);
		}
	}
	
	abstract pointcut instantiationPointcut(Object newObj);
	after(Object thisNewObj) returning(Object newObj): !isActfType() && instantiationPointcut(thisNewObj) {
		String kind = thisJoinPoint.getKind();
		if (kind.equals(JoinPoint.CONSTRUCTOR_CALL)) {
			cacheCreationContext(newObj, thisJoinPoint.getSignature(), new Exception());
		}
	}

	after(Object newObj) returning: !isActfType() && instantiationPointcut(newObj) {
		String kind = thisJoinPoint.getKind();
		if (kind.equals(JoinPoint.CONSTRUCTOR_EXECUTION) || kind.equals(JoinPoint.INITIALIZATION)) {
			cacheCreationContext(newObj, thisJoinPoint.getSignature(), new Exception());
		}
	}

}
