/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.aspects;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.Signature;

import org.eclipse.actf.validation.javapp.JavaValidationConstants;
import org.eclipse.actf.validation.reports.CreationContext;
import org.eclipse.actf.validation.reports.ValidationReport;

/**
 * the base class for aspects to be used when the validation target is hosted by 
 * the same JVM as that hosting the ACTF engine.
 *
 * @author Mike Squillace
 */
public abstract aspect IntraJvmValidationAspect extends BaseValidationAspect
{

	protected static ValidationReport currentReport;
	
	/**
	 * usually called by the AOP engine
	 */
	public IntraJvmValidationAspect (String modelName) {
		super(modelName);
	}
	
	public static ValidationReport getCurrentReport() {
		return currentReport;
	}

	public static void setCurrentReport(ValidationReport currentReport) {
		IntraJvmValidationAspect.currentReport = currentReport;
	}
	
	protected void cacheCreationContext (Object newObj, Object member, Exception exception) {
		super.cacheCreationContext(newObj, member, exception);
		if (currentReport != null) {
			CreationContext context = new CreationContext(
					getObjectId(newObj), member.toString(), exception.getStackTrace()
			);
			currentReport.addCreationContext(context);
		}
	}

	// TODO necessary until find out why aop.xml files
	// are not found on classpath in 1.4 mode
	pointcut isActfType(): within(org.eclipse.actf..*);
	
	abstract pointcut validationPointcut(Object component);
	after(Object component) returning: !isActfType() && validationPointcut(component) {
		Signature sig = thisJoinPoint.getSignature();
		StringBuffer reportID = new StringBuffer(sig.getDeclaringTypeName());
		reportID.append('@');
		reportID.append(System.identityHashCode(component));
		reportID.append('.');
		reportID.append(sig.getName());
		// TODO - validate
	}
	
	abstract pointcut instantiationPointcut(Object newObj);
	after(Object thisNewObj) returning(Object newObj): !isActfType() && instantiationPointcut(thisNewObj) {
		String kind = thisJoinPoint.getKind();
		if (kind.equals(JoinPoint.CONSTRUCTOR_CALL)) {
			cacheCreationContext(newObj, thisJoinPoint.getSignature(), new Exception());
		}
	}

	after(Object newObj) returning: !isActfType() && instantiationPointcut(newObj) {
		String kind = thisJoinPoint.getKind();
		if (kind.equals(JoinPoint.CONSTRUCTOR_EXECUTION) || kind.equals(JoinPoint.INITIALIZATION)) {
			cacheCreationContext(newObj, thisJoinPoint.getSignature(), new Exception());
		}
	}
	
	abstract pointcut terminationPointcut();
before(): terminationPointcut() {
	if (System.getProperty(JavaValidationConstants.RESOURCE_SERVER_PORT_KEY) == null) {
		throw new IllegalTerminationError("Cannot permit termination method - end session from view toolbar");
	}
}

}
