/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.events;

/**
 * A LaunchListener is used to monitor launch events generated by the <code>ValidationLauncher</code>.
 * Significant launch events occur when a GUI-based application is executed and upon the termination of
 * this execution.
 * 
 * @author Mike Squillace
 */
public interface ILaunchListener
{

	/**
	 * call-back for the initiation of a launch or execution
	 * 
	 * @param e
	 */
	public void launchInitiated (LaunchEvent e);

	/**
	 * call-back for the termination of a launch or execution
	 * 
	 * @param e
	 */
	public void launchCompleted (LaunchEvent e);

	/**
	 * call-back for determining when AOP launch has moved to
	 * a new phase. Phases include:
	 * 
	 * <p><ul>
	 * <li>preparation - cleaning temporary directory and unpacking/copying .class files
	 * <li>pre-instrumentation of .class files
	 * <li>execution of GUI-based application
	 * </ul>
	 * 
	 * @param e
	 */
	public void launchProgressed (LaunchEvent e);
}
