/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.launch;

import org.aspectj.weaver.tools.WeavingAdaptor;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.util.resources.ClassLoaderCache;
import org.eclipse.actf.validation.ValidationConstants;
import org.eclipse.actf.validation.javapp.JavaValidationConstants;
import org.eclipse.actf.validation.javapp.JavaValidationPlugin;

public class AspectJValidationLauncher extends ValidationLauncher
{

	public static final String ASPECTJ_AOP_VERBOSE_KEY = WeavingAdaptor.WEAVING_ADAPTOR_VERBOSE;

	public static final String ASPECTJ_AOP_PATH_KEY = "aj.aspect.path";

	public static final String ASPECTJ_CLASSPATH_KEY = "aj.class.path";

	//public static final double startTime = (double) System.currentTimeMillis();
	public static final String ASPECTJ_AGENT = "aspectj";

	public AspectJValidationLauncher (String executableName) {
		this(executableName, null);
	}

	public AspectJValidationLauncher (String executableName, String aopFilename) {
		this(executableName, aopFilename, ".");
	}

	public AspectJValidationLauncher (String executableName,
										String aopFilename, String workingDir) {
		super(executableName, aopFilename, workingDir);
	}

	public void prepareForLaunch () throws Exception {
		super.prepareForLaunch();
		valConfig.setSymbolPool(ValidationConstants.VALIDATION_ID);
		if (valConfig.getBooleanParameter(JavaValidationConstants.ACTF_AOP_VERBOSE_KEY)) {
			addSystemProperty(ASPECTJ_AOP_VERBOSE_KEY, "true");
			addSystemProperty(WeavingAdaptor.SHOW_WEAVE_INFO_PROPERTY, "true");
		}
		
		// form path to all AOP jars
		valConfig.setSymbolPool(IConfiguration.ACTF_ID);
		String aop = valConfig.getStringParameter(JavaValidationConstants.ACTF_AOP_ENGINE_PATH_KEY);
		String actf = valConfig.getStringParameter(ValidationConstants.ACTF_CLASSPATH_KEY);
		String os = valConfig.getStringParameter(ValidationConstants.OS_CLASSPATH_KEY);
		String interp = valConfig.getStringParameter(ValidationConstants.SCRIPTLANG_CLASSPATH_KEY);
		// for AspectJ, classpath of app being validated is in aj.class.path for 1.4 and classpath for 1.5
		String appClasspath = classpath.toString();
		if (aop == null || aop.length() == 0) {
			throw new IllegalArgumentException("No path to AspectJ AOP home directory specified");
		}else if (actf == null || aop.length() == 0) {
			throw new IllegalArgumentException("No path to ACTF class files specified"); 
		}
		classpath = new Path();
		
		IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
		
		// remove dependency on accservice
		//String msaaJar = context.getResourceLocator().getPath(AccessibilityServicePlugin.ACCESSIBILITY_MSAA_JAR);
		//String ia2Jar  = context.getResourceLocator().getPath(AccessibilityServicePlugin.ACCESSIBILITY_IA2_JAR);;
		String msaaJar = "", ia2Jar = "";
		Class pluginCls = ClassLoaderCache.getDefault().classForName("org.eclipse.actf.accservice.core.AccessibilityServicePlugin");
		if (pluginCls != null) {
			msaaJar = context.getResourceLocator().getPath((String) pluginCls.getField("ACCESSIBILITY_MSAA_JAR").get(null));
			ia2Jar = context.getResourceLocator().getPath((String) pluginCls.getField("ACCESSIBILITY_IA2_JAR").get(null));
		}
		
		if (isUsingJava14()) {
			Path ajClasspath = new Path();
			ajClasspath.append(actf);
			ajClasspath.append(msaaJar);
			ajClasspath.append(ia2Jar);
			ajClasspath.append(os);
			ajClasspath.append(interp);
			ajClasspath.append(appClasspath);
			ajClasspath.append(aopPath);
			classpath.append(aop);
			addSystemProperty(ASPECTJ_CLASSPATH_KEY, ajClasspath.toString());
			addSystemProperty(ASPECTJ_AOP_PATH_KEY, actf);
			addSystemProperty(
				"java.system.class.loader",
				"org.aspectj.weaver.loadtime.WeavingURLClassLoader");
		}else {
			bootClasspath_a = new Path(aop);
			bootClasspath_a.append(aopPath);
			classpath = new Path(appClasspath);
			classpath.append(actf);
			classpath.append(msaaJar);
			classpath.append(ia2Jar);
			classpath.append(interp);
			classpath.append(os);
			
			String javeCoreJar = context.getResourceLocator().getPath(JavaValidationPlugin.JAVA_VALIDATION_CORE_JAR);
			addJvmArgument("javaagent:", quoteString(javeCoreJar) + "=" + ASPECTJ_AGENT);	
		}
	} // prepareForLaunch
} // AspectJValidationLauncher
