/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 


package org.eclipse.actf.validation.javapp.runtime;

import java.io.File;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.AbstractRuntimeContext;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.StreamErrorLogger;
import org.eclipse.actf.util.resources.IResourceLocator;
import org.eclipse.actf.validation.javapp.JavaValidationConstants;

public class InterJvmRuntimeContext extends AbstractRuntimeContext
{

	public InterJvmRuntimeContext() {
		super();
		try {
			errorLogger = new StreamErrorLogger(System.err);
			resourceLocator = new InterJvmResourceLocator();
		
			String name = System.getProperty(
				JavaValidationConstants.RESOURCE_SERVER_HOST_KEY,
				JavaValidationConstants.RESOURCE_SERVER_HOST_DEFAULT);
			int port = Integer.parseInt(System.getProperty(
				JavaValidationConstants.RESOURCE_SERVER_PORT_KEY,
				Integer.toString(JavaValidationConstants.RESOURCE_SERVER_PORT_DEFAULT)));
		
			((InterJvmResourceLocator)resourceLocator).setHostName(name);
			((InterJvmResourceLocator)resourceLocator).setPort(port);
		} catch ( Exception e ) {
			Utils.println(IReporter.SYSTEM_FATAL, "InterJvmRuntimeContext constructor failed", e);
		}
	}

	public IConfiguration getConfiguration () throws ConfigurationException {
		if (configuration == null) {
			String resourcesPaths = System.getProperty(
					IResourceLocator.ACTF_RESOURCES_PATHS_KEY, IResourceLocator.DEFAULT_ACTF_RESOURCES_DIR);
			String[] paths = resourcesPaths.split(File.pathSeparator);
			
			Utils.println(Utils.ALL,
					"InterJvmRuntimeContext Found location of configuration files in System properties at " + resourcesPaths);
			
			try {
				configuration = ((InterJvmResourceLocator)resourceLocator).requestConfiguration();
			} catch ( Exception e ) {
				Utils.println(IReporter.SYSTEM_FATAL, "requestConfiguration failed", e);
			}
			
			/*
			configuration = new XmlConfiguration();
			for (int p = 0; p < paths.length; ++p) {
				InputStream configFileStream = resourceLocator.getResourceAsStream(
						Configuration.ACTF_ID, paths[p],
						"xml", getClass().getClassLoader()
				);
				
				if (configFileStream != null) {
					configuration.addConfigurationData(configFileStream);
				}
			}
			*/
		}
		
		return configuration;
	}

}

