/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.ui;

import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.model.ModelPlugin;
import org.eclipse.actf.util.resources.ClassLoaderCache;
import org.eclipse.actf.util.resources.EclipseResourceLocator;
import org.eclipse.actf.validation.javapp.JavaValidationPlugin;
import org.eclipse.actf.validation.ui.AbstractValidationUIPlugin;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class JavaValidationUIPlugin extends AbstractValidationUIPlugin
{

	public static final String UI_PLUGIN_ID = "org.eclipse.actf.validation.javapp.ui";

	public static final String JAVAUI_PLUGIN_RESOURCE_ID = UI_PLUGIN_ID + "Resources";

	public static final String DEFAULT_IMAGE_NAME = "actf.gif";

	public static final String UI_TRACE_FILE = "ui_trace.out";

	// The shared instance.
	private static JavaValidationUIPlugin plugin;

	public String getPluginId () {
		return UI_PLUGIN_ID;
	}

	public String getResourceBundleId () {
		return JAVAUI_PLUGIN_RESOURCE_ID;
	}

	protected String getTraceOptionId () {
		return "org.eclipse.actf.validation.javapp.ui/debug/trace";
	}

	protected String getDebugOptionId () {
		return "org.eclipse.actf.validation.javapp.ui/debug";
	}

	protected String getLogOptionId () {
		return "org.eclipse.actf.validation.javapp.ui/debug/log";
	}
	
	protected String getDefaultImageName() {
		return DEFAULT_IMAGE_NAME;
	}
	
	protected String getValidationTraceFile() {
		return UI_TRACE_FILE;
	}

	/**
	 * The constructor.
	 */
	public JavaValidationUIPlugin () {
		super();
		plugin = this;
		/*
		 * try {
		 * InstantiationLocatorAspectDeployer.getInstance().deploy(JarPreinstrumentor.SWT_JAR_POINTCUTS); }
		 * catch (Throwable e) { logException("cannot instrument SWT", e); }
		 */
	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start (BundleContext context) throws Exception {
		IRuntimeContext runtimeContext = RuntimeContextFactory.getInstance().getRuntimeContext();
		EclipseResourceLocator locator = (EclipseResourceLocator)runtimeContext.getResourceLocator();
		// remove dependency on accservice
		//locator.registerBundleName(AccessibilityServicePlugin.ACCESSIBILITY_IA2_BUNDLE);
		//locator.registerBundleName(AccessibilityServicePlugin.ACCESSIBILITY_MSAA_BUNDLE);	
		Class pluginCls = ClassLoaderCache.getDefault().classForName("org.eclipse.actf.accservice.core.AccessibilityServicePlugin");
		if (pluginCls != null) {
			locator.registerBundleName((String) pluginCls.getField("ACCESSIBILITY_MSAA_BUNDLE").get(null));
			locator.registerBundleName((String) pluginCls.getField("ACCESSIBILITY_IA2_BUNDLE").get(null));
		}

		locator.registerBundleName(JavaValidationPlugin.ASPECTJ_RUNTIME_BUNDLE);
		locator.registerBundleName(JavaValidationPlugin.ASPECTJ_WEAVER_BUNDLE);
		locator.registerBundleName(ModelPlugin.MODEL_PLUGIN_ID);
		JavaValidationPlugin.getDefault();
		// Be sure to register bundles and load plugins before starting
		// to be sure resources can be found
		super.start(context);
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop (BundleContext context) throws Exception {
		super.stop(context);
	}

	public Shell getShell () {
		Shell result = null;
		IWorkbenchWindow window = getDefault().getWorkbench().getActiveWorkbenchWindow();
		if (window != null) {
			result = window.getShell();
		}
		return result;
	}

	/**
	 * Returns the shared instance.
	 */
	public static JavaValidationUIPlugin getDefault () {
		return plugin;
	}

}
