/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.ui.actions;

import org.eclipse.actf.validation.javapp.ui.JavaValidationUIPlugin;
import org.eclipse.actf.validation.javapp.ui.validate.JavaValidationJob;
import org.eclipse.actf.validation.javapp.ui.views.RenderedGuiView;
import org.eclipse.actf.validation.ui.ValidationUIConstants;
import org.eclipse.actf.validation.ui.actions.AbstractAction;
import org.eclipse.actf.validation.ui.jobs.ValidationJob;
import org.eclipse.actf.validation.ui.views.AbstractViewPart;
import org.eclipse.core.resources.IResource;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;

public class JavaValidateAction extends AbstractAction
{

	public void run (IAction action) {
		IWorkbenchPage wbPage = activePart.getSite().getPage();
		IStructuredSelection selection = (IStructuredSelection) wbPage.getSelection();
		IJavaElement selected = selection != null ? (IJavaElement) selection.getFirstElement() : null;
		RenderedGuiView rgView = (RenderedGuiView) AbstractViewPart.activateView(RenderedGuiView.RENDERED_GUI_VIEW_ID);
		JavaValidationJob job = rgView != null ? (JavaValidationJob) rgView.getValidationJob() : null;
		
		if (selected != null
				&& (job == null || job.getJavaElement() != selected)) {
			if (job != null && job.getJavaElement() != null) {
				try {
					job.getJavaElement().getUnderlyingResource().deleteMarkers(
						ValidationUIConstants.VALIDATION_MARKER_ID, true, IResource.DEPTH_ZERO);
				}catch (Exception e) {
					JavaValidationUIPlugin.getDefault().logException(e);
				}
			}
			
			try {
				job = new JavaValidationJob(JavaValidationJob.JAVA_VALIDATION_JOB_NAME);
				job.reset(selected);
				job.setSourcePart(rgView);
			}catch (Exception e) {
				MessageDialog.openInformation(
					PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
					"Error in Java Validation",
					e.getMessage() == null ? "Error finding constructor/method for selected Java element" : e.getMessage()
				);
				plugin.logException(e);
				e.printStackTrace();
				job = null;
			}
		}else if (selected != null) {
			job.setProperty(ValidationJob.NEW_ACTION_QNAME, Boolean.FALSE);
		}
		
		if (job != null && rgView != null) {
			rgView.setValidationJob(job);
			job.schedule();
		}
	}

	public void setActivePart (IAction action, IWorkbenchPart part) {
		activePart = part;
	}
}
