/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.ui.dialogs;

import org.eclipse.actf.validation.javapp.launch.ValidationLauncher;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IconAndMessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class LaunchProgressIndicator extends IconAndMessageDialog
{

	private ValidationLauncher _launcher = null;

	private Text _text;

	private Display _display;

	public LaunchProgressIndicator (Shell shell) {
		super(shell);
		setShellStyle(SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(false);
	}

	protected Image getImage () {
		return null;
	}

	protected boolean isStarted;

	public void start () {
		isStarted = true;
	}

	public boolean isStarted () {
		return isStarted;
	}

	public String getMessage () {
		return message;
	}

	/**
	 * Set the message, blocking call.
	 */
	public void setMessage (final String message) {
		if (_text != null && !_text.isDisposed()) {
			_display.asyncExec(new Runnable() {

				public void run () {
					_text.append(message);
					_text.redraw();
				}
			});
		}
	}

	protected Control createDialogArea (Composite parent) {
		_display = getShell().getDisplay();
		final Composite area = new Composite(parent, SWT.NULL);
		// We use a grid layout and set the size of the margins
		final GridLayout gridLayout = new GridLayout();
		gridLayout.marginWidth = 15;
		gridLayout.marginHeight = 10;
		area.setLayout(gridLayout);
		_text = new Text(area, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL);
		final GridData gridData = new GridData();
		gridData.widthHint = 200;
		gridData.heightHint = 200;
		_text.setLayoutData(gridData);
		return area;
	}

	protected void createButtonsForButtonBar (Composite parent) {
		Button close_b = createButton(
			parent, IDialogConstants.CLOSE_ID, IDialogConstants.CLOSE_LABEL,
			true);
		close_b.addSelectionListener(new SelectionListener() {

			public void widgetSelected (SelectionEvent e) {
				close();
			}

			public void widgetDefaultSelected (
												org.eclipse.swt.events.SelectionEvent e) {
			}
		});
		Button cancel_b = createButton(
			parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL,
			true);
		cancel_b.addSelectionListener(new SelectionListener() {

			public void widgetSelected (SelectionEvent e) {
				Process p = _launcher != null ? _launcher.getProcess() : null;
				if (p != null) {
					p.destroy();
					_launcher.setProcess(null);
				}
			}

			public void widgetDefaultSelected (SelectionEvent e) {
			}
		});
	}

	protected void configureShell (Shell shell) {
		super.configureShell(shell);
		shell.setText("Launch Progress Indicator");
	}

	public int open () {
		return open(false);
	}

	public int open (boolean start) {
		int res = super.open();
		if (start) {
			start();
		}
		return res;
	}

	public int open (boolean start, ValidationLauncher launcher) {
		int res = super.open();
		_launcher = launcher;
		if (start) {
			start();
		}
		return res;
	}

	public boolean close () {
		Shell shell = getShell();
		if (shell != null && !shell.isDisposed()) {
			shell.setCursor(null);
		}
		return super.close();
	}
}
