/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.ui.preferences;

import org.eclipse.actf.ui.preferences.AbstractBasePreferencePage;
import org.eclipse.actf.validation.javapp.ui.JavaValidationUIPlugin;
import org.eclipse.actf.validation.ui.preferences.PathEditor;
import org.eclipse.actf.validation.ui.preferences.PropertiesEditor;
import org.eclipse.actf.validation.ui.preferences.TabFolderLayout;
import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.RadioGroupFieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * @author Mike Squillace
 */
public class AopPreferencePage extends AbstractBasePreferencePage
	implements IWorkbenchPreferencePage
{

	private PropertiesEditor aopPropertyEditor;

	private PathEditor aopPathEditor;

	private RadioGroupFieldEditor aopContextEditor;

	private BooleanFieldEditor aopVerboseEditor;

	public AopPreferencePage () {
		setPreferenceStore(JavaValidationUIPlugin.getDefault().getPreferenceStore());
	} // end--AopPreferencePage

	public void init (IWorkbench workbench) {
	} // end--init

	/* (non-Javadoc)
	 * Method declared on PreferencePage.
	 */
	protected Control createContents (Composite parent) {
		initializeDialogUnits(parent);
		// create the TabFolder Object
		TabFolder folder = new TabFolder(parent, SWT.NONE);
		folder.setLayout(new TabFolderLayout());
		folder.setLayoutData(new GridData(GridData.FILL_BOTH));
		// folder.setFont(parent.getFont());
		// create each individual tab
		// create the Properties tab
		TabItem item = new TabItem(folder, SWT.NONE);
		item.setText("Properties"); //$NON-NLS-1$;
		// item.setData(this);
		item.setControl(createAOPPropertyTab(folder));
		// create the Paths tab
		item = new TabItem(folder, SWT.NONE);
		item.setText("AOP Paths"); //$NON-NLS-1$;
		// item.setData(this);
		item.setControl(createAOPPathTab(folder));
		// create the Paths tab
		item = new TabItem(folder, SWT.NONE);
		item.setText("Options"); //$NON-NLS-1$;
		item.setControl(createAOPOptionsTab(folder));
		// load up the initial values from the system files into each tabs' contents
		initialize();
		return folder;
	}

	private Control createAOPPropertyTab (Composite parent) {
		// Create the standard tab container/layout
		Composite stdComposite = createStandardLayout(parent, 2);
		// add a properites editor to this tab container/layout
		aopPropertyEditor = new PropertiesEditor(JavaValidationPreferenceInitializer.P_SYSTEM_PROPERTIES, "Additional system properties:", stdComposite);
		return stdComposite;
	} // end--createAOPPropertyTab

	private Control createAOPPathTab (Composite parent) {
		// Create the standard tab container/layout
		Composite stdComposite = createStandardLayout(parent, 2);
		// add a path editor to this tab container/layout
		aopPathEditor = new PathEditor(JavaValidationPreferenceInitializer.P_AOP_ASPECTPATH, "AOP aspect path:", stdComposite);
		return stdComposite;
	} // end--createAOPPathTab

	private Control createAOPOptionsTab (Composite parent) {
		// Create the standard tab container/layout
		Composite stdComposite = createStandardLayout(parent, 0);
		// add editors to this tab container/layout
		aopContextEditor = new RadioGroupFieldEditor(JavaValidationPreferenceInitializer.P_CREATION_CONTEXTS_PROCESSING, "Process creation contexts for:", 2, JavaValidationPreferenceInitializer.ccChoices, stdComposite, true);
		aopVerboseEditor = new BooleanFieldEditor(JavaValidationPreferenceInitializer.P_AOP_VERBOSE, "Verbose AOP processing", stdComposite);
		// Create an exception container/layout
		// NOTE:  BooleanFieldEditor creates some funkiness with the layout so you must put the next
		// editor or item after it, in the following layout/container
		Composite exComposite = createExceptionLayout(stdComposite);
		return stdComposite;
	} // end--createAOPEngineTab

	public void initialize () {
		// Add the initial content to the property editor
		initializeEditor(this, aopPropertyEditor, getPreferenceStore());
		// Add the initial content to the path editor
		initializeEditor(this, aopPathEditor, getPreferenceStore());
		// Add the initial content to the Context radio group editor
		initializeEditor(this, aopContextEditor, getPreferenceStore());
		// Add the initial content to the Verbose checkbox editor
		initializeEditor(this, aopVerboseEditor, getPreferenceStore());
	} // end--initialize

	/* (non-Javadoc)
	 * Method declared on PreferencePage.
	 * 
	 * When the Restore Defaults button is pressed, call to load the default back into the appropriate editor
	 */
	protected void performDefaults () {
		aopPropertyEditor.loadDefault();
		aopPathEditor.loadDefault();
		aopContextEditor.loadDefault();
		aopVerboseEditor.loadDefault();
	} // end--performDefaults

	/* (non-Javadoc)
	 * Method declared on PreferencePage.
	 * 
	 * When the Ok button is pressed, call to store the contents of each editor into the system files
	 */
	public boolean performOk () {
		aopPropertyEditor.store();
		aopPathEditor.store();
		aopContextEditor.store();
		aopVerboseEditor.store();
		return super.performOk();
	} // end--performOk
} // end--AopPreferencePage
