/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.ui.validate;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.CodeSource;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import org.aspectj.weaver.ExtensibleURLClassLoader;
import org.aspectj.weaver.tools.WeavingAdaptor;
import org.aspectj.weaver.tools.WeavingClassLoader;
import org.eclipse.actf.core.ActfCorePlugin;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.model.ModelPlugin;
import org.eclipse.actf.model.javapp.JavaModelPlugin;
import org.eclipse.actf.validation.ValidationCorePlugin;
import org.eclipse.actf.validation.javapp.JavaValidationPlugin;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import org.osgi.framework.Bundle;

public class IntraJvmWeavingClassLoader extends ExtensibleURLClassLoader
	implements WeavingClassLoader
{

	// TODO used to be oriented around actual names of archives but this causes problems
	// when plugins are not unpacked in target user installation; all of this should also be 
	// dynamically handled depending on installed models
	private static final String[] BUNDLE_NAMES = {
		ActfCorePlugin.ACTFCORE_PLUGIN_ID, 
		ModelPlugin.MODEL_PLUGIN_ID,
		JavaModelPlugin.JAVA_MODEL_PLUGIN_ID,
		"org.eclipse.actf.accservice",
		ValidationCorePlugin.VALIDATION_CORE_PLUGIN_ID,
		JavaValidationPlugin.MODELJAVACORE_PLUGIN_ID, 
		JavaValidationPlugin.ASPECTJ_RUNTIME_BUNDLE, 
		
		// TODO open-source bundles; will grow as use of Orbit grows
		"org.apache.commons.beanutils",
		"org.apache.commons.collections",
		"org.apache.commons.logging",
		"org.mozilla.javascript"
	};
	
	private URL[] _aspects;
	private WeavingAdaptor _adaptor;
	private Map _generatedClasses = new HashMap();

	public IntraJvmWeavingClassLoader (URL[] classURLs, URL[] aspectURLs, ClassLoader parent) {
		super(classURLs, parent);
		init(aspectURLs);
	}

	private void init (URL[] aspectURLs) {		
		IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
		String platformLoc = Platform.getInstallLocation().getURL().getPath();
		URL aspectLibUrl = null;
		
		for (int b = 0; b < BUNDLE_NAMES.length; ++b) {
			Bundle bundle = Platform.getBundle(BUNDLE_NAMES[b]);
			if (bundle != null) {
				Enumeration en = bundle.findEntries("/", "*.jar", true);
				while (en != null && en.hasMoreElements()) {
					try {
						URL url = FileLocator.resolve((URL) en.nextElement());
						if (url.getPath().indexOf('!') < 0) {
							// not a nested jar
							// TODO remove once aspects are generated
							if (url.getPath().endsWith("actf-aspects.jar")) {
								aspectLibUrl = url;
							}
							super.addURL(url);
						}
					} catch (IOException e) {
					}
				}
				
				if (bundle != null && en == null) {
					// bundle is packed
					String bundleLoc = bundle.getLocation();
					File file = new File(platformLoc.substring(1) + bundleLoc.substring(bundleLoc.indexOf('@') + 1));
					try {
						super.addURL(file.toURL());
					} catch (MalformedURLException e) {
					}
				}
			}
		}

		if (aspectURLs == null) {
			_aspects = new URL[] {aspectLibUrl};
		} else {
			_aspects = new URL[aspectURLs.length + 1];
			_aspects[0] = aspectLibUrl;
			System.arraycopy(aspectURLs, 0, _aspects, 1, aspectURLs.length);
		}
		
		_adaptor = new WeavingAdaptor(this);
	}

	protected void addURL (URL url) {
		_adaptor.addURL(url);
		super.addURL(url);
	}

	protected Class defineClass (String name, byte[] bytes, CodeSource source)
		throws IOException {
		return super.defineClass(name, _adaptor.weaveClass(name, bytes, true), source);
	}

	protected byte[] getBytes (String name) throws IOException {
		byte[] bytes = super.getBytes(name);
		if (bytes == null) {
			bytes = (byte[]) _generatedClasses.remove(name);
		}
		return bytes;
	}

	public URL[] getAspectURLs () {
		return _aspects;
	}

	public void acceptClass (String name, byte[] bytes) {
		_generatedClasses.put(name, bytes);
	}
	/*
	 protected String findLibrary (String name)
	 {
	 Bundle bundle = Platform.getBundle("org.eclipse.swt");
	 System.err.println("Bundle:"+bundle);
	 Enumeration urls = bundle.findEntries("/", System.mapLibraryName(name), true);
	 System.err.println("URLs:");
	 //try {
	 //while(urls.hasMoreElements())System.err.println(Platform.resolve((URL)urls.nextElement()).getFile());
	 //} catch (IOException e) {
	 // TODO Auto-generated catch block
	 //e.printStackTrace();
	 //}
	 return "c:/tempaop/"+System.mapLibraryName(name);
	 }*/
}
