/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.ui.validate;

import org.eclipse.actf.validation.javapp.ui.views.RenderedGuiView;
import org.eclipse.actf.validation.ui.jobs.ValidationJob;
import org.eclipse.jdt.core.ElementChangedEvent;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IElementChangedListener;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaElementDelta;
import org.eclipse.jdt.core.IJavaModel;

public class JavaValidationElementChangedListener
	implements IElementChangedListener
{

	private JavaElementResolver _resolver = new JavaElementResolver();

	private RenderedGuiView _rgView;

	public JavaValidationElementChangedListener (RenderedGuiView jvView) {
		_rgView = jvView;
	}

	public void elementChanged (ElementChangedEvent e) {
		IJavaElementDelta delta = e.getDelta();
		final JavaValidationJob job = (JavaValidationJob) _rgView.getValidationJob();
		if (job != null && delta.getElement() instanceof IJavaModel
				&& (delta.getFlags() & ~IJavaElementDelta.ADDED) != 0) {
			IJavaElement current = job.getJavaElement();
			boolean refresh = containsEffectedCompilationUnit(
				delta.getChangedChildren(),
				(ICompilationUnit) current.getAncestor(IJavaElement.COMPILATION_UNIT));
			if (refresh && _rgView.isAutoValidating()) {
				job.setProperty(ValidationJob.NEW_ACTION_QNAME, Boolean.TRUE);
				job.schedule(1000);
			}
		}
	}

	private boolean containsEffectedCompilationUnit (
														IJavaElementDelta[] children,
														ICompilationUnit unit) {
		boolean result = false;
		for (int c = 0; !result & c < children.length; ++c) {
			result = children[c].getElement().getElementName().equals(
				unit.getElementName())
					|| containsEffectedCompilationUnit(
						children[c].getChangedChildren(), unit);
		}
		return result;
	}
}
