/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  {INITIAL AUTHOR} - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.reports;


/**
 * embodies a single creation context for a component. A creation context
 * expresses the source context for the component's instantiation.
 * 
 * @author Mike Squillace
 */
public class CreationContext extends ReportElement {

	static final long serialVersionUID = -4500494903073755699L;

	private String componentID;
	private String member;
	private StackTraceElement[] stackTrace;

	/**
	 * create a creation context object
	 * 
	 * 
	 */
	public CreationContext() {
		this("", "", new StackTraceElement[0]);
	}

	/**
	 * create a new creation context. The id should identify the component that
	 * has been instantiated, the member should represent the signature of the
	 * constructor or some signature of a member that instantiated the object,
	 * and the trace should be the stack trace that leads to the member
	 * invocation.
	 * 
	 * @param id
	 * @param member
	 * @param trace
	 */
	public CreationContext(String id, String member, StackTraceElement[] trace) {
		componentID = id;
		this.member = member;
		stackTrace = trace;
	}

	public String getComponentID() {
		return componentID;
	}

	public void setComponentID(String id) {
		componentID = id;
	}

	public String getMember() {
		return member;
	}

	public void setMember(String member) {
		this.member = member;
	}

	public StackTraceElement[] getStackTrace() {
		return stackTrace;
	}

	public void setStackTrace(StackTraceElement[] trace) {
		stackTrace = trace;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(getMember());
		sb.append(" [");
		sb.append(getStackTrace().length > 0 ? getStackTrace()[0].toString() : "");
		sb.append("...]\n");
		return sb.toString();
	}
} // CreationContext
