/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.rulebase;

import java.net.URL;

import org.eclipse.actf.model.IModel;
import org.eclipse.actf.validation.doc.codeproc.ICodeProcessor;

/**
 * interface embodying a rulebase expressed as an XML document. See the
 * "Validation ReflectionValidationDocument Authoring Guide" for details on the structure and syntax
 * of a validation document.
 * 
 * <p>
 * <b>Note</b>: The document rulebase is currently the only rulebase supported
 * by ACTF.
 * 
 * @author John Buslawski
 * 
 */
public interface IDocumentRuleBase extends IRuleBase {

	/**
	 * model pool id for aliases - value is 'aliases'
	 */
	public static final String ALIASES_ID = "aliases";

	/**
	 * Retrieve the URL of the document used for building the IRuleBase.
	 * 
	 * @return URL of the validation document
	 */
	public URL getDocumentLocation();

	/**
	 * Retrieve the ICodeProcessor for this IRuleBase. The ICodeProcessor is
	 * derived by the &lt;actf:scriptlang&gt; attribute of the root &lt;raven:rulebase&gt;
	 * element.
	 * 
	 * @return ICodeProcessor instance for the IRuleBase
	 */
	public ICodeProcessor getCodeProcessor();

	/**
	 * Retrieve the model for this IRuleBase. The model is derived
	 * by the &lt;raven:model&gt; attribute of the root &lt;raven:rulebase&gt;
	 * element.
	 * 
	 * @return model instance for the IRuleBase
	 */
	public IModel getModel();

	/**
	 * Set the ICodeProcessor for this IRuleBase
	 * 
	 * @param codeProc -
	 *            ICodeProcessor instance for the IRuleBase
	 */
	public void setCodeProcessor(ICodeProcessor codeProc);

	/**
	 * Set the model for this IRuleBase
	 * 
	 * @param model -
	 *            IModel instance for the IRuleBase
	 */
	public void setModel(IModel model);
	
}
