/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.rulebase;

import java.net.URL;
import java.util.Properties;

import org.eclipse.actf.validation.doc.rulebase.ScriptElement;



/**
 * implemented to provide metadata for a document rulebase. This will contain
 * (at the very least) the script processor used by the rulebase.
 * 
 * @author John Buslawski
 * 
 */
public interface IDocumentRuleBaseMetadata extends IRuleBaseMetadata {
	
	public static final String SCRIPTS = "scripts";
	public static final String ALIASES = "aliases";

	/**
	 * @return URL of document or <code>null</code> if no name is specified
	 */
	public URL getDocumentLocation();

	/**
	 * @param location
	 *            URL of document
	 */
	public void setDocumentLocation(URL location);

	/**
	 * retreave the scripts associated with a document
	 * 
	 * @return list of script elements or an empty list if no script elements
	 *         are present
	 */
	public ScriptElement[] getScripts();

	/**
	 * set the list of scripts to be associated with a document. Scripts are
	 * defined in the document with the &lt;raven:scripts&gt; element.
	 * 
	 * <p>
	 * Scripts will typically be executed by a <code>ICodeProcessor</code>
	 * object.
	 * 
	 * @param scripts
	 *            is a list of script elements or an empty list if no script
	 *            elements are present
	 * @see org.eclipse.actf.validation.doc.codeproc.ICodeProcessor
	 */
	public void setScripts(ScriptElement[] scripts);

	/**
	 * get the aliases associated with a document
	 * 
	 * @return map of alias name/value pairs or an empty map if no alias
	 *         elements are present
	 * @see #setAliases(Properties)
	 */
	public Properties getAliases();

	/**
	 * Get the alias value for the given alias name.
	 * 
	 * @param aliasName --
	 *            alias name
	 * @return value of alias or <code>null</code> if no alias is defined for
	 *         the given name
	 */
	public String getAliasValue(String aliasName);

	/**
	 * retreave the aliases for a document. Aliases are name-value pairs for
	 * convenient reference to fully-qualified names. Aliases are defined using
	 * the &lt;raven:alias&gt; element in a validation document.
	 * 
	 * @param aliases
	 *            is a map of alias name/value pairs
	 */
	public void setAliases(Properties aliases);

}
