/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.rules;

import java.util.List;
import java.util.Map;

/**
 * basic map for mapping components to the validation rules that are to be
 * applied to them
 * 
 * @author John Buslawski
 * 
 */
public interface IRuleMapping {

	/**
	 * Returns the complete map of Rules. The rule names are the keys into the
	 * Map while the values are the corresponding IRule objects.
	 * 
	 * @return the map of Rules for all components
	 * 
	 */
	public Map getRules();

	/**
	 * Sets the complete map of Rules. The rule names are the keys into the Map
	 * while the values are the corresponding IRule objects.
	 * 
	 * @param rules
	 *            the map of Rules for all components
	 * 
	 */
	public void setRules(Map rules);

	/**
	 * Retrieve the list of Rules to be used to validate the component. The list
	 * contains a set of rules to be validated for components of a given type
	 * including their parents if the component was marked as polymorphic.
	 * 
	 * @param id -
	 *            the id of the component (may be <code>null</code>)
	 * @param component -
	 *            component being validated
	 * @return a list of lists of rules to be validated for the given component
	 */
	public List getRulesForComponent(String id, Object component);
}
