/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.actions;

import org.eclipse.actf.validation.ui.ValidationUiPlugin;
import org.eclipse.actf.validation.ui.views.ReportView;
import org.eclipse.jface.action.IAction;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;


public class ExportReportAction extends AbstractAction
{

	public ExportReportAction () {
		super();
	}

	//	 Export to either XML or Txt format
	public void run (IAction action) {
		if (view instanceof ReportView) {
			ReportView rv = (ReportView) view;
			// first check to be sure if there are any lines in the report
			// otherwise 
			// no point in prompting for file dialog if there is no content
			if (rv.checkForContent()) {
				// Create File Dialog
				Shell shell = ValidationUiPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell();
				FileDialog fd = new FileDialog(shell, SWT.SAVE);
				// Allow them to export to XML or to TXT
				fd.setFilterNames(new String[] {"XML", "Text(*.txt)"});
				fd.setFilterExtensions(new String[] {"*.xml", "*.txt"});
				String file = fd.open();
				if (file != null && file.length() > 0) {
					//if file.ext == XML
					if (file.endsWith(".xml")) {
						rv.exportXML(file);
						//else if file.ext == TXT   
						return;
					}else if (file.endsWith(".txt")) {
						rv.exportText(file);
						return;
					}else { //should never get here - log error
						plugin.logError("Invalid format specified.  Export must be to XML or plain text (TXT) format.");
						return;
					}
				}
			}else {
				plugin.logError("Report has no data to export");
				return;
			}
		}
	}
}
