/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.dialogs;

import org.eclipse.actf.validation.reports.ValidationReport;
import org.eclipse.actf.validation.ui.views.ReportManagerView;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;


public class ReportPropertiesDialog extends TitleAreaDialog
{

	ValidationReport report = null;

	ReportManagerView view = null;

	//	 controls
	Label idLabel = null;

	Text idText = null;

	// TODO move to reports project 
	/*Label userLabel = null;

	Text userText = null;*/

	Label descriptionLabel = null;

	Text descriptionText = null;

	Label timestampLabel = null;

	Text timestampText = null;

	//TODO remove for now - rework next release
	/* Label	archLabel		= null;
	 Text	archText		= null;*/
	Label toolLabel = null;

	Text toolText = null;

	Label sourceLabel = null;

	Text sourceText = null;

	Button activeButton = null;

	Button savedButton = null;

	boolean active = true;

	String description = null;

	boolean persistent = false;

	public ReportPropertiesDialog (Shell parentShell, ValidationReport report,
									ReportManagerView rv) {
		super(parentShell);
		this.report = report;
		view = rv;
		active = report.isActive();
		description = report.getDescription();
		persistent = report.isPersistent();
	}

	protected Control createContents (Composite parent) {
		Control contents = super.createContents(parent);
		setMessage(
			"View the properties of the validation report.",
			IMessageProvider.INFORMATION);
		setTitle("Report Properties");
		return contents;
	}

	protected Control createDialogArea (Composite parent) {
		Composite area = (Composite) super.createDialogArea(parent);
		Composite controls = new Composite(area, SWT.NONE);
		GridData gd = new GridData(GridData.FILL_BOTH);
		controls.setLayoutData(gd);
		GridLayout layout = new GridLayout(2, false);
		controls.setLayout(layout);
		//report ID 
		idLabel = new Label(controls, SWT.LEFT);
		idLabel.setText("ID:");
		idText = new Text(controls, SWT.READ_ONLY);
		idText.setText(report.getReportID());
		//TODO move to reports project
		// user
		/*userLabel = new Label(controls, SWT.LEFT);
		userLabel.setText("Created by:");
		userText = new Text(controls, SWT.READ_ONLY);
		String userid = report.getProperty(ReportServices.USERID_KEY);
		if (userid != null) {
			userText.setText(userid);
		}else {
			userText.setText(ReportManagerView.NOT_PERSISTENT);
		}*/
		//description 
		descriptionLabel = new Label(controls, SWT.LEFT);
		descriptionLabel.setText("Description:");
		descriptionText = new Text(controls, SWT.SINGLE);
		descriptionText.setText(description);
		//timestamp
		timestampLabel = new Label(controls, SWT.LEFT);
		timestampLabel.setText("Creation time:");
		timestampText = new Text(controls, SWT.READ_ONLY);
		timestampText.setText(report.getTimestamp().toString());
		//timestampText.setText(report.getProperty(ReportServices.TIMESTAMP_KEY));
		//TODO - remove for now - rework next release
		/* //architecture
		 archLabel = new Label(controls, SWT.LEFT);
		 archLabel.setText("GUI architecture:");
		 archText = new Text(controls,SWT.READ_ONLY);
		 String arch = report.getProperty(ReportServices.MODEL_KEY);
		 if (arch!= null) {
		 archText.setText(arch);
		 }*/
		// tool
		toolLabel = new Label(controls, SWT.LEFT);
		toolLabel.setText("Created with:");
		toolText = new Text(controls, SWT.READ_ONLY);
		toolText.setText(report.getTool());
		//toolText.setText(report.getProperty(ReportServices.TOOL_KEY));
		// source
		sourceLabel = new Label(controls, SWT.LEFT);
		sourceLabel.setText("On resource:");
		sourceText = new Text(controls, SWT.READ_ONLY);
		sourceText.setText(report.getSource());
		// is report active or not - editable control
		// if report is not persistent, inactive does not apply - disable
		activeButton = new Button(controls, SWT.CHECK);
		activeButton.setText("Active");
		if (persistent) {
			activeButton.setSelection(active);
		}else {
			activeButton.setEnabled(false);
		}
		return area;
	}

	protected void okPressed () {
		//check for changes and save them
		boolean a = activeButton.getSelection();
		report.setActive(a);
		//TODO - move this function to DB
		/*if (a != active) {
			if (a) {
				report.setProperty(ReportServices.ACTIVE_KEY, "T");
				view.markReportActive(report);
			}else {
				report.setProperty(ReportServices.ACTIVE_KEY, "F");
				view.markReportInactive(report);
			}
		}*/
		String desc = descriptionText.getText();
		if (!description.equals(desc)) {
			report.setDescription(descriptionText.getText());
			//TODO address
			//view.updateReportDescription(report);
		}
		super.okPressed();
	}
}
