/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.filters;

import org.eclipse.actf.validation.reports.ValidationReportRecord;
import org.eclipse.actf.validation.ui.providers.ISeverityProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;


public class ReportTableViewerSorter extends ViewerSorter
{

	public static final int SEVERITY = 0;

	public static final int DESCRIPTION = 1;

	public static final int TYPE = 2;

	public static final int ID = 3;

	protected ISeverityProvider sevProvider;

	private int sortingField = 0;

	private boolean ascending = true;

	public ReportTableViewerSorter (ISeverityProvider sp) {
		sevProvider = sp;
	}

	public boolean isSorterProperty (Object element, String property) {
		return true;
	}

	public int category (Object element) {
		String result = ((ValidationReportRecord) element).getLevelAsString();
		int cat = -1;
		if (result != null) {
			String[] sevKeys = sevProvider.getSeverityKeys();
			for (int i = 0; cat < 0 && i < sevKeys.length; i++) {
				if (sevKeys[i].equals(result)) {
					cat = sevKeys.length - i;
				}
			}
		}
		return cat;
	}

	public int compare (Viewer viewer, Object e1, Object e2) {
		String s1 = null, s2 = null;
		int i1;
		int i2;
		ValidationReportRecord r1 = (ValidationReportRecord) e1;
		ValidationReportRecord r2 = (ValidationReportRecord) e2;
		switch (sortingField) {
		case SEVERITY:
			i1 = category(e1);
			i2 = category(e2);
			return performIntCompare(i2, i1);//i1, i2
		case ID:
			s1 = r1.getNodeId();
			s2 = r2.getNodeId();
			return performStringCompare(s1, s2);
		case DESCRIPTION:
			s1 = r1.getMessage();
			s2 = r2.getMessage();
			return performStringCompare(s1, s2);
		case TYPE:
			s1 = r1.getNodeName();
			s2 = r2.getNodeName();
			return performStringCompare(s1, s2);
		}
		return 0;
	}

	private int performIntCompare (int i1, int i2) {
		if (ascending) {
			return i1 - i2;
		}else {
			return i2 - i1;
		}
	}

	private int performStringCompare (String s1, String s2) {
		if (ascending) {
			return s1.compareTo(s2);
		}else {
			return s2.compareTo(s1);
		}
	}

	/**
	 * @param sortingField The sortingField to set.
	 */
	public void setSortingField (int sortingField) {
		if (this.sortingField == sortingField) {
			ascending = !ascending;
		}else {
			ascending = true;
		}
		this.sortingField = sortingField;
	}

	public void setSortingField (int sortBy, boolean direction) {
		this.sortingField = sortBy;
		ascending = direction;
	}
}
