/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.preferences;

import org.eclipse.actf.ui.preferences.AbstractBasePreferencePage;
import org.eclipse.actf.validation.ui.ValidationUiPlugin;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;


/**
 * @author Mike Squillace
 */
public class LanguagePreferencePage extends AbstractBasePreferencePage
	implements IWorkbenchPreferencePage
{

	private PropertiesEditor langEditor;

	private PathEditor classpathEditor;

	public LanguagePreferencePage () {
		setPreferenceStore(ValidationUiPlugin.getDefault().getPreferenceStore());
	} // end--LanguagePreferencePage

	public void init (IWorkbench workbench) {
	} // end--init

	/* (non-Javadoc)
	 * Method declared on PreferencePage.
	 */
	protected Control createContents (Composite parent) {
		initializeDialogUnits(parent);
		// create the Language Container
		createLanguageContainer(parent);
		// load up the initial values from the system files into each tabs' contents
		initialize();
		return parent;
	}

	private Control createLanguageContainer (Composite parent) {
		// Create the standard tab container/layout
		Composite stdComposite = createStandardLayout(parent, 2);
		// add a properites editor to this tab container/layout
		langEditor = new PropertiesEditor(ValidationPreferenceInitializer.P_SCRIPTLANGS, "Supported languages:", stdComposite);
		classpathEditor = new PathEditor(ValidationPreferenceInitializer.P_CODE_PROCESSOR_CLASSPATH, "Code Processor Classpath:", stdComposite);
		return stdComposite;
	} // end--createLanguageContainer

	public void initialize () {
		// Add the initial content to the property and path editors
		initializeEditor(this, langEditor, getPreferenceStore());
		initializeEditor(this, classpathEditor, getPreferenceStore());
	} // end--initialize

	/* (non-Javadoc)
	 * Method declared on PreferencePage.
	 * 
	 * When the Restore Defaults button is pressed, call to load the default back into the appropriate editor
	 */
	protected void performDefaults () {
		langEditor.loadDefault();
		classpathEditor.loadDefault();
	} // end--performDefaults

	/* (non-Javadoc)
	 * Method declared on PreferencePage.
	 * 
	 * When the Ok button is pressed, call to store the contents of each editor into the system files
	 */
	public boolean performOk () {
		langEditor.store();
		classpathEditor.store();
		return super.performOk();
	} // end--performOk
} // end--LanguagePreferencePage
