/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.preferences;

import org.eclipse.actf.validation.ui.ValidationUiPlugin;
import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.ColorFieldEditor;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;


/**
 * @author Mike Squillace
 */
public class ValidationPreferencePage extends PreferencePage implements
		IWorkbenchPreferencePage {

	

	private BooleanFieldEditor useDefaultBkgdColorEditor;

	private ColorFieldEditor errorBkgdColorEditor;

	private ColorFieldEditor warnBkgdColorEditor;

	private ColorFieldEditor okBkgdColorEditor;

	public ValidationPreferencePage() {
		super();
		setPreferenceStore(ValidationUiPlugin.getDefault().getPreferenceStore());
	}

	public void init(IWorkbench workbench) {
	}

	protected Control createContents(Composite parent) {
		TabFolder folder = new TabFolder(parent, SWT.NONE);
		folder.setLayout(new TabFolderLayout());
		folder.setLayoutData(new GridData(GridData.FILL_BOTH));
		TabItem item = new TabItem(folder, SWT.NONE);
		item.setText("Display");
		item.setControl(createDisplayTab(folder));
		initialize();
		return folder;
	}

	private Control createDisplayTab(Composite parent) {
		GridData gd = null;
		Composite displayComposite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		displayComposite.setLayout(layout);
		gd = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		displayComposite.setLayoutData(gd);
		useDefaultBkgdColorEditor = new BooleanFieldEditor(
				ValidationPreferenceInitializer.P_COLORS_NORMAL,
				"Use Default Background Color", displayComposite);
		Composite bkgdComposite = new Composite(displayComposite, SWT.NONE);
		GridLayout bkgdLayout = new GridLayout(2, false);
		bkgdComposite.setLayout(bkgdLayout);
		gd = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.GRAB_HORIZONTAL);
		gd.horizontalSpan = 3;
		bkgdComposite.setLayoutData(gd);
		errorBkgdColorEditor = new ColorFieldEditor(
				ValidationPreferenceInitializer.P_COLOR_ERROR,
				"Error Background Color:", bkgdComposite);
		errorBkgdColorEditor.getColorSelector().getButton().setToolTipText(
				"Select the error background color");
		warnBkgdColorEditor = new ColorFieldEditor(
				ValidationPreferenceInitializer.P_COLOR_WARNING,
				"Warning Background Color:", bkgdComposite);
		warnBkgdColorEditor.getColorSelector().getButton().setToolTipText(
				"Select the warning background color");
		okBkgdColorEditor = new ColorFieldEditor(
				ValidationPreferenceInitializer.P_COLOR_OK, "Ok Background Color:",
				bkgdComposite);
		okBkgdColorEditor.getColorSelector().getButton().setToolTipText(
				"Select the ok background color");
		return displayComposite;
	}

	private void initialize() {
		initializeDisplay();
	}

	private void initializeDisplay() {
		useDefaultBkgdColorEditor.setPreferenceStore(getPreferenceStore());
		useDefaultBkgdColorEditor.load();
		errorBkgdColorEditor.setPreferenceStore(getPreferenceStore());
		errorBkgdColorEditor.load();
		warnBkgdColorEditor.setPreferenceStore(getPreferenceStore());
		warnBkgdColorEditor.load();
		okBkgdColorEditor.setPreferenceStore(getPreferenceStore());
		okBkgdColorEditor.load();
	}

	protected void performDefaults() {
		
		performDisplayDefaults();
		super.performDefaults();
	}

	

	protected void performDisplayDefaults() {
		useDefaultBkgdColorEditor.loadDefault();
		errorBkgdColorEditor.loadDefault();
		warnBkgdColorEditor.loadDefault();
		okBkgdColorEditor.loadDefault();
	}

	public boolean performOk() {
		
		performDisplayOk();
		return super.performOk();
	}

	

	protected void performDisplayOk() {
		useDefaultBkgdColorEditor.store();
		errorBkgdColorEditor.store();
		warnBkgdColorEditor.store();
		okBkgdColorEditor.store();
	}
}
