/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.providers;

import org.eclipse.actf.validation.reports.ValidationReport;
import org.eclipse.actf.validation.reports.ValidationReportGroup;
import org.eclipse.actf.validation.reports.ValidationReportRecord;
import org.eclipse.actf.validation.ui.ValidationUIConstants;
import org.eclipse.actf.validation.ui.views.ReportManagerView;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;


public class ReportManagerLabelProvider extends LabelProvider
	implements ILabelProvider
{


	private Image reportGroupIcon = null;

	private Image reportIcon = null;

	private Image recordIcon = null;

	public ReportManagerLabelProvider (ReportManagerView view) {
		reportIcon = view.getRegisteredImage(ValidationUIConstants.REPORT_ICON_ID);
		recordIcon = view.getRegisteredImage(ValidationUIConstants.RECORD_ICON_ID);
		reportGroupIcon = view.getRegisteredImage(ValidationUIConstants.REPORT_GROUP_ICON);
	}

	public String getText (Object element) {
		String text = "";
		if (element instanceof ValidationReportGroup) {
			text = "Report Group";
		}else if (element instanceof ValidationReport) {
			text = ((ValidationReport) element).getSource()
					+ " ["
					+ ((ValidationReport) element).getTimestamp().toString()
					+ "]";
		}else if (element instanceof ValidationReportRecord) {
			text = ((ValidationReportRecord) element).getMessage();
		}
		return text;
	}

	public Image getImage (Object element) {
		Image image = null;
		if (element instanceof ValidationReportGroup) {
			image = reportGroupIcon;
		}else if (element instanceof ValidationReport) {
				image = reportIcon;
		}else if (element instanceof ValidationReportRecord) {
			image = recordIcon;
		}
		return image;
	}

	public void dispose () {
		// clean up the images
		
		if (reportGroupIcon != null) {
			reportGroupIcon.dispose();
		}
		if (reportIcon != null) {
			reportIcon.dispose();
		}
		
		if (recordIcon != null) {
			recordIcon.dispose();
		}
	}
}
