package org.eclipse.actf.validation.ui.providers;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.actf.validation.doc.rulebase.ValidationDocumentRuleSet;
import org.eclipse.actf.validation.rulebase.IRuleBase;
import org.eclipse.actf.validation.rules.ComponentCriteria;
import org.eclipse.actf.validation.rules.IRule;
import org.eclipse.actf.validation.rules.IRuleMapping;
import org.eclipse.actf.validation.rules.IRuleSet;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

public class RuleBaseContentProvider implements ITreeContentProvider
{
	Map components;
	Map compCriteriaMap;
	
	public RuleBaseContentProvider () {
		super();
	}
	
	public Object[] getChildren (Object parentElement) {
		System.out.println("getChildren");
		Object []object = new Object[0];
		if(parentElement instanceof IRuleBase) {
			System.out.println("\t Instanceof IRuleBase");
			IRuleSet []ruleSets = ((IRuleBase)parentElement).getRuleSets();
			object = ruleSets;
		} else if(parentElement instanceof IRuleSet) {
			System.out.println("\t Instanceof IRuleSet");
			ValidationDocumentRuleSet vdocRuleSet = (ValidationDocumentRuleSet) parentElement;
			IRuleMapping rMapping = vdocRuleSet.getRuleMap();
			components = rMapping.getRules();
			Set compKeys = components.keySet();
			System.out.println("compKeys : "+compKeys);
			object = new Object[compKeys.size()];
			int counter = 0;
			for(Iterator compKeysItr = compKeys.iterator();compKeysItr.hasNext();) {
				object[counter] = (compKeysItr.next());
				++counter;
			}
		} else if(parentElement instanceof ComponentCriteria) {
			System.out.println("parentElement instanceof ComponentCriteria");
			List rulesList = (List)compCriteriaMap.get((ComponentCriteria)parentElement);
			if(rulesList != null) {
				object = rulesList.toArray();
			} 				
		} else if(parentElement instanceof IRule) {
			System.out.println("Instanceof IRule");
			List ruleParts = ((IRule)parentElement).getRuleParts();
			object = ruleParts.toArray();
		} else {
			if(!components.isEmpty() && components.containsKey(parentElement)) {
					System.out.println("Didnot match in getChildren()");
					compCriteriaMap = (HashMap)components.get(parentElement);
					Set compCritKeys = compCriteriaMap.keySet();
					ArrayList<Object> compCritList = new ArrayList<Object>();
					for(Iterator compCritItr = compCritKeys.iterator(); compCritItr.hasNext();) {
						compCritList.add(compCritItr.next());
					}
					object = compCritList.toArray();
			}
		}
		System.out.println("\t object : "+object);
		return object;
	}

	public Object getParent (Object element) {
		Object obj = null;
		if(element instanceof IRuleBase) {
			obj = null;
		} else if(element instanceof IRuleSet) {
			obj = ((IRuleSet)element).getRuleBase();
		} else if(element instanceof IRule) {
			obj = ((IRule)element).getParent();
		} else {
			System.out.println("getParent: Don't know what to do returning null obj");
			obj = null;
		}
		return obj;
	}

	public boolean hasChildren (Object element) {
		Object[] obj = getChildren(element);
		boolean result =  obj == null ? false : obj.length>0;
		return result;
	}

	public Object[] getElements (Object inputElement) {
		ArrayList rBases = (ArrayList)inputElement;
		return rBases.toArray();
	}

	public void dispose () {
		//Nothing to dispose
	}

	public void inputChanged (Viewer viewer, Object oldInput, Object newInput) {
		//Nothing to change
	}
	
}
