/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  barryf - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.views;

import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.validation.ui.preferences.ValidationPreferenceInitializer;
import org.eclipse.actf.validation.ui.providers.BaseViewPartLabelProvider;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ContentViewer;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.Widget;

/**
 * @author barryf
 * 
 */
abstract public class BaseViewPart extends AbstractViewPart
	implements PaintListener
{

	Control control;

	public Control getViewControl () {
		return control;
	}

	protected Object active;

	public Object getActiveObject () {
		return active;
	}

	protected void setActiveObject (Object o) {
		active = o;
	}

	ContentViewer contentViewer;

	public ContentViewer getViewer () {
		return contentViewer;
	}

	Color outlineFG;

	// Color newFG;
	Color newBG;

	Color oldFG;

	Color oldBG;

	Control oldControl;

	/**
	 * 
	 */
	public BaseViewPart () {
	}

	protected boolean useMarkerColor = true;

	public boolean useMarkerColor () {
		return useMarkerColor;
	}

	public void setUseMarkerColor (Control parent, boolean f) {
		useMarkerColor = f;
		if (useMarkerColor) {
			createColors(parent);
		}else {
			cleanup();
		}
	}

	public Color createColor (Control ctl, String key) {
		RGB rgb = PreferenceConverter.getColor(getPreferenceStore(), key);
		Color c = new Color(ctl.getDisplay(), rgb);
		trace(IReporter.DETAIL, "createColor: " + ctl + "," + c);
		return c;
	}

	protected void createColors (Control ctl) {
		cleanup();
		trace(IReporter.DETAIL, "createColors: " + ctl + "," + useMarkerColor);
		if (useMarkerColor && ctl != null) {
			// outlineFG = new Color(control.getDisplay(), 0, 255, 0);
			// newFG = new Color(control.getDisplay(), 100, 150, 175);
			// newBG = new Color(control.getDisplay(), 255, 200, 225);
			outlineFG = createColor(
				ctl, ValidationPreferenceInitializer.P_MARKER_OUTLINE_COLOR);
			newBG = createColor(
				ctl, ValidationPreferenceInitializer.P_MARKER_BG_COLOR);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl (final Composite parent) {
		super.createPartControl(parent);
		contentViewer.setContentProvider(getContentProvider());
		contentViewer.setLabelProvider(getLabelProvider());
		IPreferenceStore store = getPreferenceStore();
		useMarkerColor = store.getBoolean(ValidationPreferenceInitializer.P_USE_MARKER_COLOR);
		createColors(parent);
		store.addPropertyChangeListener(new IPropertyChangeListener() {

			public void propertyChange (PropertyChangeEvent e) {
				try {
					String property = e.getProperty();
					if (property.equals(ValidationPreferenceInitializer.P_USE_MARKER_COLOR)) {
						Object o = e.getNewValue();
						if (o instanceof Boolean) {
							setUseMarkerColor(
								parent,
								((Boolean) e.getNewValue()).booleanValue());
						}else if (o instanceof String) {
							setUseMarkerColor(
								parent,
								((String) e.getNewValue()).equals("true"));
						}
					}else if (property.equals(ValidationPreferenceInitializer.P_MARKER_BG_COLOR)
							|| property.equals(ValidationPreferenceInitializer.P_MARKER_OUTLINE_COLOR)) {
						createColors(parent);
					}
				}catch (Exception ex) {
					logException("cannot change color property", ex);
				}
			}
		});
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPart#setFocus()
	 */
	public void setFocus () {
		if (control != null) {
			control.setFocus();
		}
	}

	public void setInput (Object c) {
		setInput(c, true);
	}

	public void resetInput () {
		if (active != null) {
			setInput(active);
		}
	}

	public void resetInput (boolean expand) {
		if (active != null) {
			setInput(active, expand);
		}
	}

	public abstract void setInput (Object c, boolean expand);

	protected void cleanup () {
		trace(IReporter.DETAIL, "cleanup");
		if (outlineFG != null && !outlineFG.isDisposed()) {
			outlineFG.dispose();
			outlineFG = null;
		}
		if (newBG != null && !newBG.isDisposed()) {
			newBG.dispose();
			newBG = null;
		}
	}

	public void dispose () {
		cleanup();
		super.dispose();
		control = null;
	}

	public void paintControl (PaintEvent e) {
		// trace("paintControl: " + e);
		if (outlineFG != null && !outlineFG.isDisposed()) {
			e.gc.setForeground(outlineFG);
			e.gc.drawFocus(e.x, e.y, e.width - 1, e.height - 1);
		}
	}

	protected class ViewPartSelectionListener extends XSelectionAdapter
	{

	
		public void widgetSelected (SelectionEvent e) {
			// trace("widgetSelected: " + e);
			doSelection(e);
		}
	}

	protected SelectionListener getViewPartSelectionListener () {
		return new ViewPartSelectionListener();
	}

	protected Object selected;

	public Object getLastSelected () {
		return selected;
	}

	protected void setLastSelected (Object selected) {
		this.selected = selected;
	}

	protected void doSelection (SelectionEvent e) {
		Object o = e.item;
		deselect(o);
		select(o);
	}

	protected class FocusHandler implements FocusListener
	{

		protected Object lastSelected;

		/**
		 * @return Returns the lastSelected.
		 */
		public Object getSelected () {
			return lastSelected;
		}

		/**
		 * @param lastSelected
		 *            The lastSelected to set.
		 */
		public void setSelected (Object lastSelected) {
			this.lastSelected = lastSelected;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.events.FocusListener#focusGained(org.eclipse.swt.events.FocusEvent)
		 */
		public void focusGained (FocusEvent e) {
			if (lastSelected != null) {
				select(lastSelected);
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.events.FocusListener#focusLost(org.eclipse.swt.events.FocusEvent)
		 */
		public void focusLost (FocusEvent e) {
			if (lastSelected != null) {
				deselect(lastSelected);
			}
		}
	}

	protected FocusHandler focusHandler = new FocusHandler();

	protected void deselect (Object o) {
		if (oldControl != null && !oldControl.isDisposed()) {
			try {
				focusHandler.setSelected(null);
				oldControl.removePaintListener(this);
				oldControl.setBackground(oldBG);
				// oldControl.setForeground(oldFG);
				oldControl.redraw();
			}catch (SWTException e) {
			}
		}
		oldControl = null;
		// setLastSelected(null);
	}

	protected void select (Object o) {
		// trace("select: " + o);
		if (o != null && o instanceof Widget && !((Widget) o).isDisposed()) {
			if (o instanceof Item) {
				Item ti = (Item) o;
				String titext = ti.getText();
				if (titext.startsWith("@")) {
					String xcode = titext.substring(1, titext.indexOf(':'));
					int code = Integer.parseInt(xcode, 16);
					Control xc = findControl(getViewControl().getShell(), code);
					if (xc != null) {
						setLastSelected(o);
						focusHandler.setSelected(o);
						oldControl = xc;
						oldControl.addPaintListener(this);
						// oldFG = xc.getForeground();
						oldBG = xc.getBackground();
						// if (newFG != null && !newFG.isDisposed()) {
						// xc.setForeground(newFG);
						// }
						if (newBG != null && !newBG.isDisposed()) {
							xc.setBackground(newBG);
							xc.redraw();
						}
					}
				}
			}
		}
	}

	protected static String truncate (String s, int len) {
		if (s.length() > len) {
			s = s.substring(0, len / 2 - 1) + "..."
					+ s.substring(s.length() - len / 2 - 2);
		}
		return s;
	}

	protected static String truncate (String s) {
		return truncate(s, 100);
	}

	protected IBaseLabelProvider getLabelProvider () {
		return new BaseViewPartLabelProvider(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.actf.validation.ui.views.AbstractTreeView#getTreeContentProvider()
	 */
	protected IContentProvider getContentProvider () {
		//This is just a placeholder - the real value will come from the subclass
		return new ArrayContentProvider();
	}
}
