package org.eclipse.actf.validation.ui.views;

import java.util.ArrayList;

import org.eclipse.actf.validation.rulebase.DocumentRuleBaseDescriptor;
import org.eclipse.actf.validation.rulebase.IRuleBase;
import org.eclipse.actf.validation.rulebase.RuleBaseManager;
import org.eclipse.actf.validation.ui.providers.RuleBaseContentProvider;
import org.eclipse.actf.validation.ui.providers.RuleBaseLabelProvider;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

public class RuleBaseView extends AbstractViewPart
{
	private static CheckboxTreeViewer tv = null;
	private RuleBaseViewer dialog;	
	
	public RuleBaseView () {
		super();
	}
	
	public void setFocus () {
		tv.getControl().setFocus();		
	}
	
	public static CheckboxTreeViewer getCheckBoxTreeViewer() {
		return tv;
	}
	
	protected String getMessageTitle () {
		return "RuleBase Viewer View";
	}

	public void createPartControl (Composite parent) {
//		Uncomment to load the SWT and SWING rulebases too, this should include 
//		the java.core plugin dependency to work.
//		try {
//			Class.forName("org.eclipse.actf.validation.javapp.core.JavaValidationPlugin");
//		} catch (ClassNotFoundException e) {
//			e.printStackTrace();
//		}
		
//		Another way to load plugin without dependency but works for known plugins,
//		TODO: Figure out a way for unknown plugins or the ones which are not in the platform
//		Need to take care of plugin not found exceptions.
//		Platform.getPlugin("org.eclipse.actf.validation.javapp.core.JavaValidationPlugin");
		super.createPartControl(parent);
		final Composite composite = new Composite(parent,SWT.NONE);
		composite.setLayout(new GridLayout(3,false));
		tv = new CheckboxTreeViewer(composite);
		tv.getTree().setLayoutData(new GridData(GridData.FILL_BOTH));
		tv.setContentProvider(new RuleBaseContentProvider());
		tv.setLabelProvider(new RuleBaseLabelProvider());		
		tv.setInput(getRuleBases());
		tv.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged (SelectionChangedEvent event) {
				Object object = ((TreeSelection)event.getSelection()).getFirstElement();
				dialog.updateRule(object);
			}
		});
		
		tv.addCheckStateListener(new ICheckStateListener() {
			public void checkStateChanged (CheckStateChangedEvent event) {
				if(event.getChecked())
					tv.setSubtreeChecked(event.getElement(), true);
				else
					tv.setSubtreeChecked(event.getElement(), false);
				}
		});
		dialog = new RuleBaseViewer(composite);		
	}

	private ArrayList getRuleBases() {
		RuleBaseManager rbMgr = RuleBaseManager.getInstance();
		ArrayList<IRuleBase> ruleBases = new ArrayList<IRuleBase>();
		for (int i = 0; i < rbMgr.getRuleBaseDescriptors().length; i++) {
			DocumentRuleBaseDescriptor rbDesc = (DocumentRuleBaseDescriptor) rbMgr.getRuleBaseDescriptors()[i];
			String rbName = rbDesc.getRuleBaseName();
			System.out.println("rbName : "+rbName);
			System.out.println("rb Source Location : "+rbDesc.getSourceLocation());
			IRuleBase rBase = rbMgr.getRuleBase(rbName);
			ruleBases.add(rBase);
		}
		return ruleBases;
	}
}
