package org.eclipse.actf.validation.ui.views;



import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.validation.rules.IRule;
import org.eclipse.actf.validation.rules.RuleConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class RuleBaseViewer
{
	//Widgets variables
	private Text longDescText;
	private Text messageText;
	private Text IDText;
	private Text repairHintText;
	private Text compCategoryText;
	private Combo severityCombo;
	private Button ignoreIfNullButton;
	private Button polymorphicButton;
	private Composite parent;

	public RuleBaseViewer(Composite composite) {
		initGUI(composite);
	}

	public void hide() {
		parent.setVisible(false);
	}

	public void unHide() {
		parent.setVisible(true);
	}

	private void initGUI (final Composite composite) {
		parent = new Composite(composite,SWT.NONE);
		parent.setLayout(new GridLayout(2,false));
		hide();
		longDescText = createLabelText(parent, "Long Description", "");
		messageText = createLabelText(parent, "Message", "");
		IDText = createLabelText(parent, "ID", "");
		repairHintText = createLabelText(parent, "Repair Hint", "");
		severityCombo = createComboBox(parent, "Severity", IReporter.KEYS, IReporter.DEFAULT_LEVEL);
		Composite checkBoxes = createButtonComposite(parent, 2);
		ignoreIfNullButton = createButton(checkBoxes, SWT.CHECK, "IgnoreIfNull", false);
		polymorphicButton = createButton(checkBoxes, SWT.CHECK, "Polymorphic", false);
		compCategoryText = createLabelText(parent, "Component Category", "");
	}

	//update contenets
	public void updateRule(Object ruleObject) {
		if(ruleObject instanceof IRule) {
			unHide();
			IRule rule = (IRule)ruleObject;
			longDescText.setText(rule.getLongDescription() == null ? "":rule.getLongDescription());
			messageText.setText(rule.getMessage() == null ? "":rule.getMessage());
			IDText.setText(rule.getId() == null ? "":rule.getId());
			repairHintText.setText(rule.getRepairHint() == null ? "":rule.getRepairHint());
			severityCombo.select(rule.getSeverity() == 0 ? 4:rule.getSeverity());
			ignoreIfNullButton.setSelection(rule.getIgnoreIfNull());
			polymorphicButton.setSelection(Boolean.parseBoolean(rule.getAttribute(RuleConstants.POLYMORPHIC)));
			compCategoryText.setText(rule.getComponentCategory() == null? "":rule.getComponentCategory());
		} else {
			hide();
		}
	}

	//Creates a label
	private Label createLabel (Composite comp, String labelValue) {
		Label label = new Label(comp, SWT.NONE);
		label.setText(labelValue);
		return label;
	}

	//Creates a label and textBox associated with it
	private Text createLabelText (Composite comp, String labelValue,
									String textValue) {
		createLabel(comp, labelValue);
		Text text = new Text(comp, SWT.BORDER);
		if(textValue == null)
			textValue = "";
		else
			text.setText(textValue);
		GridData data = new GridData();
		data.horizontalAlignment = GridData.FILL;
		text.setLayoutData(data);
		return text;
	}

	//Creates a comboBox with items and the one selected
	private Combo createComboBox (Composite comp, String labelValue,
									String[] items, int selection) {
		createLabel(comp, labelValue);
		Combo combo = new Combo(comp, SWT.DROP_DOWN);
		combo.setItems(items);
		combo.select(selection);
		GridData data = new GridData();
		data.horizontalAlignment = GridData.FILL;
		combo.setLayoutData(data);
		return combo;
	}

	//Creates composite button
	private Composite createButtonComposite (Composite parent, int numCols) {
		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout gridLayout = new GridLayout(numCols, false);
		gridLayout.makeColumnsEqualWidth = true;
		GridData data = new GridData(GridData.HORIZONTAL_ALIGN_CENTER);
		data.horizontalSpan = 2;
		composite.setLayout(gridLayout);
		composite.setLayoutData(data);
		return composite;
	}

	private Button createButton (Composite comp, int style, String buttonLabel,
									boolean isChecked) {
		GridData buttonData = new GridData(GridData.FILL_BOTH);
		Button button = new Button(comp, SWT.CHECK);
		button.setSelection(isChecked);
		button.setText(buttonLabel);
		button.setLayoutData(buttonData);
		return button;
	}
}
