/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.views;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.validation.ValidationConstants;
import org.eclipse.actf.validation.ui.providers.RuntimeLabelProvider;
import org.eclipse.actf.validation.ui.providers.RuntimeToggle;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.part.ViewPart;

public class RuntimeView extends ViewPart
{

	private CheckboxTableViewer cbtViewer;

	private Table table;

	private IConfiguration config;

	public RuntimeView () {
		super();
		try {

			IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
			config = context.getConfiguration();
			
		}catch (ConfigurationException e) {
			//Could not obtain a Configuration. We can't do much here but quit.
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	public void createPartControl (Composite parent) {
		table = new Table(parent, SWT.SINGLE | SWT.CHECK);
		table.setHeaderVisible(true);
		table.setLinesVisible(false);
		createTableColumns();
		cbtViewer = new CheckboxTableViewer(table);
		cbtViewer.setContentProvider(new ArrayContentProvider());
		cbtViewer.setLabelProvider(new RuntimeLabelProvider());
		final RuntimeToggle[] toggles = getRuntimeToggles();
		cbtViewer.setInput(toggles);
		for (int t = 0; t < toggles.length; ++t) {
			cbtViewer.setChecked(toggles[t], toggles[t].isSelected());
		}
		cbtViewer.addCheckStateListener(new ICheckStateListener() {

			public void checkStateChanged (CheckStateChangedEvent e) {
				RuntimeToggle toggle = (RuntimeToggle) e.getElement();
				boolean checked = e.getChecked();
				int index = -1;
				for (int t = 0; t < toggles.length; ++t) {
					if (toggles[t] == toggle) {
						index = t;
						break;
					}
				}
				toggle.setSelected(checked);
				config.setSymbolPool(ValidationConstants.VALIDATION_ID);
				config.setParameter(toggle.getConfigurationKey(), checked);
				table.getItem(index).setText(
					RuntimeLabelProvider.MESSAGE_COL,
					checked ? toggle.getEnabledMessage()
							: toggle.getDisabledMessage());
			}
		});
	}

	public void setFocus () {
		cbtViewer.getControl().setFocus();
	}

	public void dispose () {
		if (!table.isDisposed()) {
			table.dispose();
			table = null;
		}
		super.dispose();
	}

	protected RuntimeToggle[] getRuntimeToggles () {
		RuntimeToggle[] toggles = new RuntimeToggle[3];
		RuntimeToggle toggle = null;
		config.setSymbolPool(ValidationConstants.VALIDATION_ID);
		toggle = toggles[0] = new RuntimeToggle("Perform validation", ValidationConstants.PERFORMING_VALIDATION_KEY);
		toggle.setEnabledMessage("Validations are being performed automatically as application is used");
		toggle.setDisabledMessage("Validations are not being performed");
		toggle.setSelected(config.getBooleanParameter(ValidationConstants.PERFORMING_VALIDATION_KEY));
		toggle = toggles[1] = new RuntimeToggle("Validate hidden", ValidationConstants.VALIDATE_HIDDEN_KEY);
		toggle.setEnabledMessage("Hidden/invisible components are being validated");
		toggle.setDisabledMessage("No hidden or invisible components are being validated");
		toggle.setSelected(config.getBooleanParameter(ValidationConstants.VALIDATE_HIDDEN_KEY));
		toggle = toggles[2] = new RuntimeToggle("Revalidate components", ValidationConstants.REVALIDATING_COMPONENTS_KEY);
		toggle.setEnabledMessage("Components can be re-validated");
		toggle.setDisabledMessage("No components are being re-validated");
		toggle.setSelected(config.getBooleanParameter(ValidationConstants.REVALIDATING_COMPONENTS_KEY));
		return toggles;
	}

	protected void createTableColumns () {
		TableColumn tc = new TableColumn(table, SWT.CENTER);
		tc.setText("Label");
		tc.setWidth(150);
		tc = new TableColumn(table, SWT.LEFT);
		tc.setText("Current function");
		tc.setWidth(350);
	}
}
